"""
MAIN CLASS FOR TLS LITE (START HERE!).
"""
from __future__ import generators

import socket
from utils.compat import formatExceptionTrace
from TLSRecordLayer import TLSRecordLayer
from Session import Session
from constants import *
from utils.cryptomath import getRandomBytes
from errors import *
from messages import *
from mathtls import *
from HandshakeSettings import HandshakeSettings


class TLSConnection(TLSRecordLayer):
    """
    This class wraps a socket and provides TLS handshaking and data
    transfer.

    To use this class, create a new instance, passing a connected
    socket into the constructor.  Then call some handshake function.
    If the handshake completes without raising an exception, then a TLS
    connection has been negotiated.  You can transfer data over this
    connection as if it were a socket.

    This class provides both synchronous and asynchronous versions of
    its key functions.  The synchronous versions should be used when
    writing single-or multi-threaded code using blocking sockets.  The
    asynchronous versions should be used when performing asynchronous,
    event-based I/O with non-blocking sockets.

    Asynchronous I/O is a complicated subject; typically, you should
    not use the asynchronous functions directly, but should use some
    framework like asyncore or Twisted which TLS Lite integrates with
    (see
    L{tlslite.integration.TLSAsyncDispatcherMixIn.TLSAsyncDispatcherMixIn} or
    L{tlslite.integration.TLSTwistedProtocolWrapper.TLSTwistedProtocolWrapper}).
    """


    def __init__(self, sock):
        """Create a new TLSConnection instance.

        @param sock: The socket data will be transmitted on.  The
        socket should already be connected.  It may be in blocking or
        non-blocking mode.

        @type sock: L{socket.socket}
        """
        TLSRecordLayer.__init__(self, sock)

    def handshakeClientSRP(self, username, password, session=None,
                           settings=None, checker=None, async=False):
        """Perform an SRP handshake in the role of client.

        This function performs a TLS/SRP handshake.  SRP mutually
        authenticates both parties to each other using only a
        username and password.  This function may also perform a
        combined SRP and server-certificate handshake, if the server
        chooses to authenticate itself with a certificate chain in
        addition to doing SRP.

        TLS/SRP is non-standard.  Most TLS implementations don't
        support it.  See
        U{http://www.ietf.org/html.charters/tls-charter.html} or
        U{http://trevp.net/tlssrp/} for the latest information on
        TLS/SRP.

        Like any handshake function, this can be called on a closed
        TLS connection, or on a TLS connection that is already open.
        If called on an open connection it performs a re-handshake.

        If the function completes without raising an exception, the
        TLS connection will be open and available for data transfer.

        If an exception is raised, the connection will have been
        automatically closed (if it was ever open).

        @type username: str
        @param username: The SRP username.

        @type password: str
        @param password: The SRP password.

        @type session: L{tlslite.Session.Session}
        @param session: A TLS session to attempt to resume.  This
        session must be an SRP session performed with the same username
        and password as were passed in.  If the resumption does not
        succeed, a full SRP handshake will be performed.

        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
        @param settings: Various settings which can be used to control
        the ciphersuites, certificate types, and SSL/TLS versions
        offered by the client.

        @type checker: L{tlslite.Checker.Checker}
        @param checker: A Checker instance.  This instance will be
        invoked to examine the other party's authentication
        credentials, if the handshake completes succesfully.

        @type async: bool
        @param async: If False, this function will block until the
        handshake is completed.  If True, this function will return a
        generator.  Successive invocations of the generator will
        return 0 if it is waiting to read from the socket, 1 if it is
        waiting to write to the socket, or will raise StopIteration if
        the handshake operation is completed.

        @rtype: None or an iterable
        @return: If 'async' is True, a generator object will be
        returned.

        @raise socket.error: If a socket error occurs.
        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
        without a preceding alert.
        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
        @raise tlslite.errors.TLSAuthenticationError: If the checker
        doesn't like the other party's authentication credentials.
        """
        handshaker = self._handshakeClientAsync(srpParams=(username, password),
                        session=session, settings=settings, checker=checker)
        if async:
            return handshaker
        for result in handshaker:
            pass

    def handshakeClientCert(self, certChain=None, privateKey=None,
                            session=None, settings=None, checker=None,
                            async=False):
        """Perform a certificate-based handshake in the role of client.

        This function performs an SSL or TLS handshake.  The server
        will authenticate itself using an X.509 or cryptoID certificate
        chain.  If the handshake succeeds, the server's certificate
        chain will be stored in the session's serverCertChain attribute.
        Unless a checker object is passed in, this function does no
        validation or checking of the server's certificate chain.

        If the server requests client authentication, the
        client will send the passed-in certificate chain, and use the
        passed-in private key to authenticate itself.  If no
        certificate chain and private key were passed in, the client
        will attempt to proceed without client authentication.  The
        server may or may not allow this.

        Like any handshake function, this can be called on a closed
        TLS connection, or on a TLS connection that is already open.
        If called on an open connection it performs a re-handshake.

        If the function completes without raising an exception, the
        TLS connection will be open and available for data transfer.

        If an exception is raised, the connection will have been
        automatically closed (if it was ever open).

        @type certChain: L{tlslite.X509CertChain.X509CertChain} or
        L{cryptoIDlib.CertChain.CertChain}
        @param certChain: The certificate chain to be used if the
        server requests client authentication.

        @type privateKey: L{tlslite.utils.RSAKey.RSAKey}
        @param privateKey: The private key to be used if the server
        requests client authentication.

        @type session: L{tlslite.Session.Session}
        @param session: A TLS session to attempt to resume.  If the
        resumption does not succeed, a full handshake will be
        performed.

        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
        @param settings: Various settings which can be used to control
        the ciphersuites, certificate types, and SSL/TLS versions
        offered by the client.

        @type checker: L{tlslite.Checker.Checker}
        @param checker: A Checker instance.  This instance will be
        invoked to examine the other party's authentication
        credentials, if the handshake completes succesfully.

        @type async: bool
        @param async: If False, this function will block until the
        handshake is completed.  If True, this function will return a
        generator.  Successive invocations of the generator will
        return 0 if it is waiting to read from the socket, 1 if it is
        waiting to write to the socket, or will raise StopIteration if
        the handshake operation is completed.

        @rtype: None or an iterable
        @return: If 'async' is True, a generator object will be
        returned.

        @raise socket.error: If a socket error occurs.
        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
        without a preceding alert.
        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
        @raise tlslite.errors.TLSAuthenticationError: If the checker
        doesn't like the other party's authentication credentials.
        """
        handshaker = self._handshakeClientAsync(certParams=(certChain,
                        privateKey), session=session, settings=settings,
                        checker=checker)
        if async:
            return handshaker
        for result in handshaker:
            pass

    def handshakeClientUnknown(self, srpCallback=None, certCallback=None,
                               session=None, settings=None, checker=None,
                               async=False):
        """Perform a to-be-determined type of handshake in the role of client.

        This function performs an SSL or TLS handshake.  If the server
        requests client certificate authentication, the
        certCallback will be invoked and should return a (certChain,
        privateKey) pair.  If the callback returns None, the library
        will attempt to proceed without client authentication.  The
        server may or may not allow this.

        If the server requests SRP authentication, the srpCallback
        will be invoked and should return a (username, password) pair.
        If the callback returns None, the local implementation will
        signal a user_canceled error alert.

        After the handshake completes, the client can inspect the
        connection's session attribute to determine what type of
        authentication was performed.

        Like any handshake function, this can be called on a closed
        TLS connection, or on a TLS connection that is already open.
        If called on an open connection it performs a re-handshake.

        If the function completes without raising an exception, the
        TLS connection will be open and available for data transfer.

        If an exception is raised, the connection will have been
        automatically closed (if it was ever open).

        @type srpCallback: callable
        @param srpCallback: The callback to be used if the server
        requests SRP authentication.  If None, the client will not
        offer support for SRP ciphersuites.

        @type certCallback: callable
        @param certCallback: The callback to be used if the server
        requests client certificate authentication.

        @type session: L{tlslite.Session.Session}
        @param session: A TLS session to attempt to resume.  If the
        resumption does not succeed, a full handshake will be
        performed.

        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
        @param settings: Various settings which can be used to control
        the ciphersuites, certificate types, and SSL/TLS versions
        offered by the client.

        @type checker: L{tlslite.Checker.Checker}
        @param checker: A Checker instance.  This instance will be
        invoked to examine the other party's authentication
        credentials, if the handshake completes succesfully.

        @type async: bool
        @param async: If False, this function will block until the
        handshake is completed.  If True, this function will return a
        generator.  Successive invocations of the generator will
        return 0 if it is waiting to read from the socket, 1 if it is
        waiting to write to the socket, or will raise StopIteration if
        the handshake operation is completed.

        @rtype: None or an iterable
        @return: If 'async' is True, a generator object will be
        returned.

        @raise socket.error: If a socket error occurs.
        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
        without a preceding alert.
        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
        @raise tlslite.errors.TLSAuthenticationError: If the checker
        doesn't like the other party's authentication credentials.
        """
        handshaker = self._handshakeClientAsync(unknownParams=(srpCallback,
                        certCallback), session=session, settings=settings,
                        checker=checker)
        if async:
            return handshaker
        for result in handshaker:
            pass

    def handshakeClientSharedKey(self, username, sharedKey, settings=None,
                                 checker=None, async=False):
        """Perform a shared-key handshake in the role of client.

        This function performs a shared-key handshake.  Using shared
        symmetric keys of high entropy (128 bits or greater) mutually
        authenticates both parties to each other.

        TLS with shared-keys is non-standard.  Most TLS
        implementations don't support it.  See
        U{http://www.ietf.org/html.charters/tls-charter.html} for the
        latest information on TLS with shared-keys.  If the shared-keys
        Internet-Draft changes or is superceded, TLS Lite will track
        those changes, so the shared-key support in later versions of
        TLS Lite may become incompatible with this version.

        Like any handshake function, this can be called on a closed
        TLS connection, or on a TLS connection that is already open.
        If called on an open connection it performs a re-handshake.

        If the function completes without raising an exception, the
        TLS connection will be open and available for data transfer.

        If an exception is raised, the connection will have been
        automatically closed (if it was ever open).

        @type username: str
        @param username: The shared-key username.

        @type sharedKey: str
        @param sharedKey: The shared key.

        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
        @param settings: Various settings which can be used to control
        the ciphersuites, certificate types, and SSL/TLS versions
        offered by the client.

        @type checker: L{tlslite.Checker.Checker}
        @param checker: A Checker instance.  This instance will be
        invoked to examine the other party's authentication
        credentials, if the handshake completes succesfully.

        @type async: bool
        @param async: If False, this function will block until the
        handshake is completed.  If True, this function will return a
        generator.  Successive invocations of the generator will
        return 0 if it is waiting to read from the socket, 1 if it is
        waiting to write to the socket, or will raise StopIteration if
        the handshake operation is completed.

        @rtype: None or an iterable
        @return: If 'async' is True, a generator object will be
        returned.

        @raise socket.error: If a socket error occurs.
        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
        without a preceding alert.
        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
        @raise tlslite.errors.TLSAuthenticationError: If the checker
        doesn't like the other party's authentication credentials.
        """
        handshaker = self._handshakeClientAsync(sharedKeyParams=(username,
                        sharedKey), settings=settings, checker=checker)
        if async:
            return handshaker
        for result in handshaker:
            pass

    def _handshakeClientAsync(self, srpParams=(), certParams=(),
                             unknownParams=(), sharedKeyParams=(),
                             session=None, settings=None, checker=None,
                             recursive=False):

        handshaker = self._handshakeClientAsyncHelper(srpParams=srpParams,
                certParams=certParams, unknownParams=unknownParams,
                sharedKeyParams=sharedKeyParams, session=session,
                settings=settings, recursive=recursive)
        for result in self._handshakeWrapperAsync(handshaker, checker):
            yield result


    def _handshakeClientAsyncHelper(self, srpParams, certParams, unknownParams,
                               sharedKeyParams, session, settings, recursive):
        if not recursive:
            self._handshakeStart(client=True)

        #Unpack parameters
        srpUsername = None      # srpParams
        password = None         # srpParams
        clientCertChain = None  # certParams
        privateKey = None       # certParams
        srpCallback = None      # unknownParams
        certCallback = None     # unknownParams
        #session                # sharedKeyParams (or session)
        #settings               # settings

        if srpParams:
            srpUsername, password = srpParams
        elif certParams:
            clientCertChain, privateKey = certParams
        elif unknownParams:
            srpCallback, certCallback = unknownParams
        elif sharedKeyParams:
            session = Session()._createSharedKey(*sharedKeyParams)

        if not settings:
            settings = HandshakeSettings()
        settings = settings._filter()

        #Validate parameters
        if srpUsername and not password:
            raise ValueError("Caller passed a username but no password")
        if password and not srpUsername:
            raise ValueError("Caller passed a password but no username")

        if clientCertChain and not privateKey:
            raise ValueError("Caller passed a certChain but no privateKey")
        if privateKey and not clientCertChain:
            raise ValueError("Caller passed a privateKey but no certChain")

        if clientCertChain:
            foundType = False
            try:
                import cryptoIDlib.CertChain
                if isinstance(clientCertChain, cryptoIDlib.CertChain.CertChain):
                    if "cryptoID" not in settings.certificateTypes:
                        raise ValueError("Client certificate doesn't "\
                                         "match Handshake Settings")
                    settings.certificateTypes = ["cryptoID"]
                    foundType = True
            except ImportError:
                pass
            if not foundType and isinstance(clientCertChain,
                                            X509CertChain):
                if "x509" not in settings.certificateTypes:
                    raise ValueError("Client certificate doesn't match "\
                                     "Handshake Settings")
                settings.certificateTypes = ["x509"]
                foundType = True
            if not foundType:
                raise ValueError("Unrecognized certificate type")


        if session:
            if not session.valid():
                session = None #ignore non-resumable sessions...
            elif session.resumable and \
                    (session.srpUsername != srpUsername):
                raise ValueError("Session username doesn't match")

        #Add Faults to parameters
        if srpUsername and self.fault == Fault.badUsername:
            srpUsername += "GARBAGE"
        if password and self.fault == Fault.badPassword:
            password += "GARBAGE"
        if sharedKeyParams:
            identifier = sharedKeyParams[0]
            sharedKey = sharedKeyParams[1]
            if self.fault == Fault.badIdentifier:
                identifier += "GARBAGE"
                session = Session()._createSharedKey(identifier, sharedKey)
            elif self.fault == Fault.badSharedKey:
                sharedKey += "GARBAGE"
                session = Session()._createSharedKey(identifier, sharedKey)


        #Initialize locals
        serverCertChain = None
        cipherSuite = 0
        certificateType = CertificateType.x509
        premasterSecret = None

        #Get client nonce
        clientRandom = getRandomBytes(32)

        #Initialize acceptable ciphersuites
        cipherSuites = []
        if srpParams:
            cipherSuites += CipherSuite.getSrpRsaSuites(settings.cipherNames)
            cipherSuites += CipherSuite.getSrpSuites(settings.cipherNames)
        elif certParams:
            cipherSuites += CipherSuite.getRsaSuites(settings.cipherNames)
        elif unknownParams:
            if srpCallback:
                cipherSuites += \
                    CipherSuite.getSrpRsaSuites(settings.cipherNames)
                cipherSuites += \
                    CipherSuite.getSrpSuites(settings.cipherNames)
            cipherSuites += CipherSuite.getRsaSuites(settings.cipherNames)
        elif sharedKeyParams:
            cipherSuites += CipherSuite.getRsaSuites(settings.cipherNames)
        else:
            cipherSuites += CipherSuite.getRsaSuites(settings.cipherNames)

        #Initialize acceptable certificate types
        certificateTypes = settings._getCertificateTypes()

        #Tentatively set the version to the client's minimum version.
        #We'll use this for the ClientHello, and if an error occurs
        #parsing the Server Hello, we'll use this version for the response
        self.version = settings.maxVersion

        #Either send ClientHello (with a resumable session)...
        if session:
            #If it's a resumable (i.e. not a shared-key session), then its
            #ciphersuite must be one of the acceptable ciphersuites
            if (not sharedKeyParams) and \
                session.cipherSuite not in cipherSuites:
                raise ValueError("Session's cipher suite not consistent "\
                                 "with parameters")
            else:
                clientHello = ClientHello()
                clientHello.create(settings.maxVersion, clientRandom,
                                   session.sessionID, cipherSuites,
                                   certificateTypes, session.srpUsername)

        #Or send ClientHello (without)
        else:
            clientHello = ClientHello()
            clientHello.create(settings.maxVersion, clientRandom,
                               createByteArraySequence([]), cipherSuites,
                               certificateTypes, srpUsername)
        for result in self._sendMsg(clientHello):
            yield result

        #Get ServerHello (or missing_srp_username)
        for result in self._getMsg((ContentType.handshake,
                                  ContentType.alert),
                                  HandshakeType.server_hello):
            if result in (0,1):
                yield result
            else:
                break
        msg = result

        if isinstance(msg, ServerHello):
            serverHello = msg
        elif isinstance(msg, Alert):
            alert = msg

            #If it's not a missing_srp_username, re-raise
            if alert.description != AlertDescription.missing_srp_username:
                self._shutdown(False)
                raise TLSRemoteAlert(alert)

            #If we're not in SRP callback mode, we won't have offered SRP
            #without a username, so we shouldn't get this alert
            if not srpCallback:
                for result in self._sendError(\
                                AlertDescription.unexpected_message):
                    yield result
            srpParams = srpCallback()
            #If the callback returns None, cancel the handshake
            if srpParams == None:
                for result in self._sendError(AlertDescription.user_canceled):
                    yield result

            #Recursively perform handshake
            for result in self._handshakeClientAsyncHelper(srpParams,
                            None, None, None, None, settings, True):
                yield result
            return

        #Get the server version.  Do this before anything else, so any
        #error alerts will use the server's version
        self.version = serverHello.server_version

        #Future responses from server must use this version
        self._versionCheck = True

        #Check ServerHello
        if serverHello.server_version < settings.minVersion:
            for result in self._sendError(\
                AlertDescription.protocol_version,
                "Too old version: %s" % str(serverHello.server_version)):
                yield result
        if serverHello.server_version > settings.maxVersion:
            for result in self._sendError(\
                AlertDescription.protocol_version,
                "Too new version: %s" % str(serverHello.server_version)):
                yield result
        if serverHello.cipher_suite not in cipherSuites:
            for result in self._sendError(\
                AlertDescription.illegal_parameter,
                "Server responded with incorrect ciphersuite"):
                yield result
        if serverHello.certificate_type not in certificateTypes:
            for result in self._sendError(\
                AlertDescription.illegal_parameter,
                "Server responded with incorrect certificate type"):
                yield result
        if serverHello.compression_method != 0:
            for result in self._sendError(\
                AlertDescription.illegal_parameter,
                "Server responded with incorrect compression method"):
                yield result

        #Get the server nonce
        serverRandom = serverHello.random

        #If the server agrees to resume
        if session and session.sessionID and \
                       serverHello.session_id == session.sessionID:

            #If a shared-key, we're flexible about suites; otherwise the
            #server-chosen suite has to match the session's suite
            if sharedKeyParams:
                session.cipherSuite = serverHello.cipher_suite
            elif serverHello.cipher_suite != session.cipherSuite:
                for result in self._sendError(\
                    AlertDescription.illegal_parameter,\
                    "Server's ciphersuite doesn't match session"):
                    yield result

            #Set the session for this connection
            self.session = session

            #Calculate pending connection states
            self._calcPendingStates(clientRandom, serverRandom,
                                   settings.cipherImplementations)

            #Exchange ChangeCipherSpec and Finished messages
            for result in self._getFinished():
                yield result
            for result in self._sendFinished():
                yield result

            #Mark the connection as open
            self._handshakeDone(resumed=True)

        #If server DOES NOT agree to resume
        else:

            if sharedKeyParams:
                for result in self._sendError(\
                        AlertDescription.user_canceled,
                        "Was expecting a shared-key resumption"):
                    yield result

            #We've already validated these
            cipherSuite = serverHello.cipher_suite
            certificateType = serverHello.certificate_type

            #If the server chose an SRP suite...
            if cipherSuite in CipherSuite.srpSuites:
                #Get ServerKeyExchange, ServerHelloDone
                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.server_key_exchange, cipherSuite):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverKeyExchange = result

                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.server_hello_done):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverHelloDone = result

            #If the server chose an SRP+RSA suite...
            elif cipherSuite in CipherSuite.srpRsaSuites:
                #Get Certificate, ServerKeyExchange, ServerHelloDone
                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.certificate, certificateType):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverCertificate = result

                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.server_key_exchange, cipherSuite):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverKeyExchange = result

                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.server_hello_done):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverHelloDone = result

            #If the server chose an RSA suite...
            elif cipherSuite in CipherSuite.rsaSuites:
                #Get Certificate[, CertificateRequest], ServerHelloDone
                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.certificate, certificateType):
                    if result in (0,1):
                        yield result
                    else:
                        break
                serverCertificate = result

                for result in self._getMsg(ContentType.handshake,
                        (HandshakeType.server_hello_done,
                        HandshakeType.certificate_request)):
                    if result in (0,1):
                        yield result
                    else:
                        break
                msg = result

                certificateRequest = None
                if isinstance(msg, CertificateRequest):
                    certificateRequest = msg
                    for result in self._getMsg(ContentType.handshake,
                            HandshakeType.server_hello_done):
                        if result in (0,1):
                            yield result
                        else:
                            break
                    serverHelloDone = result
                elif isinstance(msg, ServerHelloDone):
                    serverHelloDone = msg
            else:
                raise AssertionError()


            #Calculate SRP premaster secret, if server chose an SRP or
            #SRP+RSA suite
            if cipherSuite in CipherSuite.srpSuites + \
                              CipherSuite.srpRsaSuites:
                #Get and check the server's group parameters and B value
                N = serverKeyExchange.srp_N
                g = serverKeyExchange.srp_g
                s = serverKeyExchange.srp_s
                B = serverKeyExchange.srp_B

                if (g,N) not in goodGroupParameters:
                    for result in self._sendError(\
                            AlertDescription.untrusted_srp_parameters,
                            "Unknown group parameters"):
                        yield result
                if numBits(N) < settings.minKeySize:
                    for result in self._sendError(\
                            AlertDescription.untrusted_srp_parameters,
                            "N value is too small: %d" % numBits(N)):
                        yield result
                if numBits(N) > settings.maxKeySize:
                    for result in self._sendError(\
                            AlertDescription.untrusted_srp_parameters,
                            "N value is too large: %d" % numBits(N)):
                        yield result
                if B % N == 0:
                    for result in self._sendError(\
                            AlertDescription.illegal_parameter,
                            "Suspicious B value"):
                        yield result

                #Check the server's signature, if server chose an
                #SRP+RSA suite
                if cipherSuite in CipherSuite.srpRsaSuites:
                    #Hash ServerKeyExchange/ServerSRPParams
                    hashBytes = serverKeyExchange.hash(clientRandom,
                                                       serverRandom)

                    #Extract signature bytes from ServerKeyExchange
                    sigBytes = serverKeyExchange.signature
                    if len(sigBytes) == 0:
                        for result in self._sendError(\
                                AlertDescription.illegal_parameter,
                                "Server sent an SRP ServerKeyExchange "\
                                "message without a signature"):
                            yield result

                    #Get server's public key from the Certificate message
                    for result in self._getKeyFromChain(serverCertificate,
                                                       settings):
                        if result in (0,1):
                            yield result
                        else:
                            break
                    publicKey, serverCertChain = result

                    #Verify signature
                    if not publicKey.verify(sigBytes, hashBytes):
                        for result in self._sendError(\
                                AlertDescription.decrypt_error,
                                "Signature failed to verify"):
                            yield result


                #Calculate client's ephemeral DH values (a, A)
                a = bytesToNumber(getRandomBytes(32))
                A = powMod(g, a, N)

                #Calculate client's static DH values (x, v)
                x = makeX(bytesToString(s), srpUsername, password)
                v = powMod(g, x, N)

                #Calculate u
                u = makeU(N, A, B)

                #Calculate premaster secret
                k = makeK(N, g)
                S = powMod((B - (k*v)) % N, a+(u*x), N)

                if self.fault == Fault.badA:
                    A = N
                    S = 0
                premasterSecret = numberToBytes(S)

                #Send ClientKeyExchange
                for result in self._sendMsg(\
                        ClientKeyExchange(cipherSuite).createSRP(A)):
                    yield result


            #Calculate RSA premaster secret, if server chose an RSA suite
            elif cipherSuite in CipherSuite.rsaSuites:

                #Handle the presence of a CertificateRequest
                if certificateRequest:
                    if unknownParams and certCallback:
                        certParamsNew = certCallback()
                        if certParamsNew:
                            clientCertChain, privateKey = certParamsNew

                #Get server's public key from the Certificate message
                for result in self._getKeyFromChain(serverCertificate,
                                                   settings):
                    if result in (0,1):
                        yield result
                    else:
                        break
                publicKey, serverCertChain = result


                #Calculate premaster secret
                premasterSecret = getRandomBytes(48)
                premasterSecret[0] = settings.maxVersion[0]
                premasterSecret[1] = settings.maxVersion[1]

                if self.fault == Fault.badPremasterPadding:
                    premasterSecret[0] = 5
                if self.fault == Fault.shortPremasterSecret:
                    premasterSecret = premasterSecret[:-1]

                #Encrypt premaster secret to server's public key
                encryptedPreMasterSecret = publicKey.encrypt(premasterSecret)

                #If client authentication was requested, send Certificate
                #message, either with certificates or empty
                if certificateRequest:
                    clientCertificate = Certificate(certificateType)

                    if clientCertChain:
                        #Check to make sure we have the same type of
                        #certificates the server requested
                        wrongType = False
                        if certificateType == CertificateType.x509:
                            if not isinstance(clientCertChain, X509CertChain):
                                wrongType = True
                        elif certificateType == CertificateType.cryptoID:
                            if not isinstance(clientCertChain,
                                              cryptoIDlib.CertChain.CertChain):
                                wrongType = True
                        if wrongType:
                            for result in self._sendError(\
                                    AlertDescription.handshake_failure,
                                    "Client certificate is of wrong type"):
                                yield result

                        clientCertificate.create(clientCertChain)

                    for result in self._sendMsg(clientCertificate):
                        yield result
                else:
                    #The server didn't request client auth, so we
                    #zeroize these so the clientCertChain won't be
                    #stored in the session.
                    privateKey = None
                    clientCertChain = None

                #Send ClientKeyExchange
                clientKeyExchange = ClientKeyExchange(cipherSuite,
                                                      self.version)
                clientKeyExchange.createRSA(encryptedPreMasterSecret)
                for result in self._sendMsg(clientKeyExchange):
                    yield result

                #If client authentication was requested and we have a
                #private key, send CertificateVerify
                if certificateRequest and privateKey:
                    if self.version == (3,0):
                        #Create a temporary session object, just for the
                        #purpose of creating the CertificateVerify
                        session = Session()
                        session._calcMasterSecret(self.version,
                                                 premasterSecret,
                                                 clientRandom,
                                                 serverRandom)
                        verifyBytes = self._calcSSLHandshakeHash(\
                                          session.masterSecret, "")
                    elif self.version in ((3,1), (3,2)):
                        verifyBytes = stringToBytes(\
                            self._handshake_md5.digest() + \
                            self._handshake_sha.digest())
                    if self.fault == Fault.badVerifyMessage:
                        verifyBytes[0] = ((verifyBytes[0]+1) % 256)
                    signedBytes = privateKey.sign(verifyBytes)
                    certificateVerify = CertificateVerify()
                    certificateVerify.create(signedBytes)
                    for result in self._sendMsg(certificateVerify):
                        yield result


            #Create the session object
            self.session = Session()
            self.session._calcMasterSecret(self.version, premasterSecret,
                                          clientRandom, serverRandom)
            self.session.sessionID = serverHello.session_id
            self.session.cipherSuite = cipherSuite
            self.session.srpUsername = srpUsername
            self.session.clientCertChain = clientCertChain
            self.session.serverCertChain = serverCertChain

            #Calculate pending connection states
            self._calcPendingStates(clientRandom, serverRandom,
                                   settings.cipherImplementations)

            #Exchange ChangeCipherSpec and Finished messages
            for result in self._sendFinished():
                yield result
            for result in self._getFinished():
                yield result

            #Mark the connection as open
            self.session._setResumable(True)
            self._handshakeDone(resumed=False)



    def handshakeServer(self, sharedKeyDB=None, verifierDB=None,
                        certChain=None, privateKey=None, reqCert=False,
                        sessionCache=None, settings=None, checker=None):
        """Perform a handshake in the role of server.

        This function performs an SSL or TLS handshake.  Depending on
        the arguments and the behavior of the client, this function can
        perform a shared-key, SRP, or certificate-based handshake.  It
        can also perform a combined SRP and server-certificate
        handshake.

        Like any handshake function, this can be called on a closed
        TLS connection, or on a TLS connection that is already open.
        If called on an open connection it performs a re-handshake.
        This function does not send a Hello Request message before
        performing the handshake, so if re-handshaking is required,
        the server must signal the client to begin the re-handshake
        through some other means.

        If the function completes without raising an exception, the
        TLS connection will be open and available for data transfer.

        If an exception is raised, the connection will have been
        automatically closed (if it was ever open).

        @type sharedKeyDB: L{tlslite.SharedKeyDB.SharedKeyDB}
        @param sharedKeyDB: A database of shared symmetric keys
        associated with usernames.  If the client performs a
        shared-key handshake, the session's sharedKeyUsername
        attribute will be set.

        @type verifierDB: L{tlslite.VerifierDB.VerifierDB}
        @param verifierDB: A database of SRP password verifiers
        associated with usernames.  If the client performs an SRP
        handshake, the session's srpUsername attribute will be set.

        @type certChain: L{tlslite.X509CertChain.X509CertChain} or
        L{cryptoIDlib.CertChain.CertChain}
        @param certChain: The certificate chain to be used if the
        client requests server certificate authentication.

        @type privateKey: L{tlslite.utils.RSAKey.RSAKey}
        @param privateKey: The private key to be used if the client
        requests server certificate authentication.

        @type reqCert: bool
        @param reqCert: Whether to request client certificate
        authentication.  This only applies if the client chooses server
        certificate authentication; if the client chooses SRP or
        shared-key authentication, this will be ignored.  If the client
        performs a client certificate authentication, the sessions's
        clientCertChain attribute will be set.

        @type sessionCache: L{tlslite.SessionCache.SessionCache}
        @param sessionCache: An in-memory cache of resumable sessions.
        The client can resume sessions from this cache.  Alternatively,
        if the client performs a full handshake, a new session will be
        added to the cache.

        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
        @param settings: Various settings which can be used to control
        the ciphersuites and SSL/TLS version chosen by the server.

        @type checker: L{tlslite.Checker.Checker}
        @param checker: A Checker instance.  This instance will be
        invoked to examine the other party's authentication
        credentials, if the handshake completes succesfully.

        @raise socket.error: If a socket error occurs.
        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
        without a preceding alert.
        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
        @raise tlslite.errors.TLSAuthenticationError: If the checker
        doesn't like the other party's authentication credentials.
        """
        for result in self.handshakeServerAsync(sharedKeyDB, verifierDB,
                certChain, privateKey, reqCert, sessionCache, settings,
                checker):
            pass


    def handshakeServerAsync(self, sharedKeyDB=None, verifierDB=None,
                             certChain=None, privateKey=None, reqCert=False,
                             sessionCache=None, settings=None, checker=None):
        """Start a server handshake operation on the TLS connection.

        This function returns a generator which behaves similarly to
        handshakeServer().  Successive invocations of the generator
        will return 0 if it is waiting to read from the socket, 1 if it is
        waiting to write to the socket, or it will raise StopIteration
        if the handshake operation is complete.

        @rtype: iterable
        @return: A generator; see above for details.
        """
        handshaker = self._handshakeServerAsyncHelper(\
            sharedKeyDB=sharedKeyDB,
            verifierDB=verifierDB, certChain=certChain,
            privateKey=privateKey, reqCert=reqCert,
            sessionCache=sessionCache, settings=settings)
        for result in self._handshakeWrapperAsync(handshaker, checker):
            yield result


    def _handshakeServerAsyncHelper(self, sharedKeyDB, verifierDB,
                             certChain, privateKey, reqCert, sessionCache,
                             settings):

        self._handshakeStart(client=False)

        if (not sharedKeyDB) and (not verifierDB) and (not certChain):
            raise ValueError("Caller passed no authentication credentials")
        if certChain and not privateKey:
            raise ValueError("Caller passed a certChain but no privateKey")
        if privateKey and not certChain:
            raise ValueError("Caller passed a privateKey but no certChain")

        if not settings:
            settings = HandshakeSettings()
        settings = settings._filter()

        #Initialize acceptable cipher suites
        cipherSuites = []
        if verifierDB:
            if certChain:
                cipherSuites += \
                    CipherSuite.getSrpRsaSuites(settings.cipherNames)
            cipherSuites += CipherSuite.getSrpSuites(settings.cipherNames)
        if sharedKeyDB or certChain:
            cipherSuites += CipherSuite.getRsaSuites(settings.cipherNames)

        #Initialize acceptable certificate type
        certificateType = None
        if certChain:
            try:
                import cryptoIDlib.CertChain
                if isinstance(certChain, cryptoIDlib.CertChain.CertChain):
                    certificateType = CertificateType.cryptoID
            except ImportError:
                pass
            if isinstance(certChain, X509CertChain):
                certificateType = CertificateType.x509
            if certificateType == None:
                raise ValueError("Unrecognized certificate type")

        #Initialize locals
        clientCertChain = None
        serverCertChain = None #We may set certChain to this later
        postFinishedError = None

        #Tentatively set version to most-desirable version, so if an error
        #occurs parsing the ClientHello, this is what we'll use for the
        #error alert
        self.version = settings.maxVersion

        #Get ClientHello
        for result in self._getMsg(ContentType.handshake,
                                   HandshakeType.client_hello):
            if result in (0,1):
                yield result
            else:
                break
        clientHello = result

        #If client's version is too low, reject it
        if clientHello.client_version < settings.minVersion:
            self.version = settings.minVersion
            for result in self._sendError(\
                  AlertDescription.protocol_version,
                  "Too old version: %s" % str(clientHello.client_version)):
                yield result

        #If client's version is too high, propose my highest version
        elif clientHello.client_version > settings.maxVersion:
            self.version = settings.maxVersion

        else:
            #Set the version to the client's version
            self.version = clientHello.client_version

        #Get the client nonce; create server nonce
        clientRandom = clientHello.random
        serverRandom = getRandomBytes(32)

        #Calculate the first cipher suite intersection.
        #This is the 'privileged' ciphersuite.  We'll use it if we're
        #doing a shared-key resumption or a new negotiation.  In fact,
        #the only time we won't use it is if we're resuming a non-sharedkey
        #session, in which case we use the ciphersuite from the session.
        #
        #Given the current ciphersuite ordering, this means we prefer SRP
        #over non-SRP.
        for cipherSuite in cipherSuites:
            if cipherSuite in clientHello.cipher_suites:
                break
        else:
            for result in self._sendError(\
                    AlertDescription.handshake_failure):
                yield result

        #If resumption was requested...
        if clientHello.session_id and (sharedKeyDB or sessionCache):
            session = None

            #Check in the sharedKeys container
            if sharedKeyDB and len(clientHello.session_id)==16:
                try:
                    #Trim off zero padding, if any
                    for x in range(16):
                        if clientHello.session_id[x]==0:
                            break
                    self.allegedSharedKeyUsername = bytesToString(\
                                            clientHello.session_id[:x])
                    session = sharedKeyDB[self.allegedSharedKeyUsername]
                    if not session.sharedKey:
                        raise AssertionError()
                    #use privileged ciphersuite
                    session.cipherSuite = cipherSuite
                except KeyError:
                    pass

            #Then check in the session cache
            if sessionCache and not session:
                try:
                    session = sessionCache[bytesToString(\
                                               clientHello.session_id)]
                    if session.sharedKey:
                        raise AssertionError()
                    if not session.resumable:
                        raise AssertionError()
                    #Check for consistency with ClientHello
                    if session.cipherSuite not in cipherSuites:
                        for result in self._sendError(\
                                AlertDescription.handshake_failure):
                            yield result
                    if session.cipherSuite not in clientHello.cipher_suites:
                        for result in self._sendError(\
                                AlertDescription.handshake_failure):
                            yield result
                    if clientHello.srp_username:
                        if clientHello.srp_username != session.srpUsername:
                            for result in self._sendError(\
                                    AlertDescription.handshake_failure):
                                yield result
                except KeyError:
                    pass

            #If a session is found..
            if session:
                #Set the session
                self.session = session

                #Send ServerHello
                serverHello = ServerHello()
                serverHello.create(self.version, serverRandom,
                                   session.sessionID, session.cipherSuite,
                                   certificateType)
                for result in self._sendMsg(serverHello):
                    yield result

                #From here on, the client's messages must have the right version
                self._versionCheck = True

                #Calculate pending connection states
                self._calcPendingStates(clientRandom, serverRandom,
                                       settings.cipherImplementations)

                #Exchange ChangeCipherSpec and Finished messages
                for result in self._sendFinished():
                    yield result
                for result in self._getFinished():
                    yield result

                #Mark the connection as open
                self._handshakeDone(resumed=True)
                return


        #If not a resumption...

        #TRICKY: we might have chosen an RSA suite that was only deemed
        #acceptable because of the shared-key resumption.  If the shared-
        #key resumption failed, because the identifier wasn't recognized,
        #we might fall through to here, where we have an RSA suite
        #chosen, but no certificate.
        if cipherSuite in CipherSuite.rsaSuites and not certChain:
            for result in self._sendError(\
                    AlertDescription.handshake_failure):
                yield result

        #If an RSA suite is chosen, check for certificate type intersection
        #(We do this check down here because if the mismatch occurs but the
        # client is using a shared-key session, it's okay)
        if cipherSuite in CipherSuite.rsaSuites + \
                          CipherSuite.srpRsaSuites:
            if certificateType not in clientHello.certificate_types:
                for result in self._sendError(\
                        AlertDescription.handshake_failure,
                        "the client doesn't support my certificate type"):
                    yield result

            #Move certChain -> serverCertChain, now that we're using it
            serverCertChain = certChain


        #Create sessionID
        if sessionCache:
            sessionID = getRandomBytes(32)
        else:
            sessionID = createByteArraySequence([])

        #If we've selected an SRP suite, exchange keys and calculate
        #premaster secret:
        if cipherSuite in CipherSuite.srpSuites + CipherSuite.srpRsaSuites:

            #If there's no SRP username...
            if not clientHello.srp_username:

                #Ask the client to re-send ClientHello with one
                for result in self._sendMsg(Alert().create(\
                        AlertDescription.missing_srp_username,
                        AlertLevel.warning)):
                    yield result

                #Get ClientHello
                for result in self._getMsg(ContentType.handshake,
                        HandshakeType.client_hello):
                    if result in (0,1):
                        yield result
                    else:
                        break
                clientHello = result

                #Check ClientHello
                #If client's version is too low, reject it (COPIED CODE; BAD!)
                if clientHello.client_version < settings.minVersion:
                    self.version = settings.minVersion
                    for result in self._sendError(\
                          AlertDescription.protocol_version,
                          "Too old version: %s" % str(clientHello.client_version)):
                        yield result

                #If client's version is too high, propose my highest version
                elif clientHello.client_version > settings.maxVersion:
                    self.version = settings.maxVersion

                else:
                    #Set the version to the client's version
                    self.version = clientHello.client_version

                #Recalculate the privileged cipher suite, making sure to
                #pick an SRP suite
                cipherSuites = [c for c in cipherSuites if c in \
                                CipherSuite.srpSuites + \
                                CipherSuite.srpRsaSuites]
                for cipherSuite in cipherSuites:
                    if cipherSuite in clientHello.cipher_suites:
                        break
                else:
                    for result in self._sendError(\
                            AlertDescription.handshake_failure):
                        yield result

                #Get the client nonce; create server nonce
                clientRandom = clientHello.random
                serverRandom = getRandomBytes(32)

                #The username better be there, this time
                if not clientHello.srp_username:
                    for result in self._sendError(\
                            AlertDescription.illegal_parameter,
                            "Client resent a hello, but without the SRP"\
                            " username"):
                        yield result


            #Get username
            self.allegedSrpUsername = clientHello.srp_username

            #Get parameters from username
            try:
                entry = verifierDB[self.allegedSrpUsername]
            except KeyError:
                for result in self._sendError(\
                        AlertDescription.unknown_srp_username):
                    yield result
            (N, g, s, v) = entry

            #Calculate server's ephemeral DH values (b, B)
            b = bytesToNumber(getRandomBytes(32))
            k = makeK(N, g)
            B = (powMod(g, b, N) + (k*v)) % N

            #Create ServerKeyExchange, signing it if necessary
            serverKeyExchange = ServerKeyExchange(cipherSuite)
            serverKeyExchange.createSRP(N, g, stringToBytes(s), B)
            if cipherSuite in CipherSuite.srpRsaSuites:
                hashBytes = serverKeyExchange.hash(clientRandom,
                                                   serverRandom)
                serverKeyExchange.signature = privateKey.sign(hashBytes)

            #Send ServerHello[, Certificate], ServerKeyExchange,
            #ServerHelloDone
            msgs = []
            serverHello = ServerHello()
            serverHello.create(self.version, serverRandom, sessionID,
                               cipherSuite, certificateType)
            msgs.append(serverHello)
            if cipherSuite in CipherSuite.srpRsaSuites:
                certificateMsg = Certificate(certificateType)
                certificateMsg.create(serverCertChain)
                msgs.append(certificateMsg)
            msgs.append(serverKeyExchange)
            msgs.append(ServerHelloDone())
            for result in self._sendMsgs(msgs):
                yield result

            #From here on, the client's messages must have the right version
            self._versionCheck = True

            #Get and check ClientKeyExchange
            for result in self._getMsg(ContentType.handshake,
                                      HandshakeType.client_key_exchange,
                                      cipherSuite):
                if result in (0,1):
                    yield result
                else:
                    break
            clientKeyExchange = result
            A = clientKeyExchange.srp_A
            if A % N == 0:
                postFinishedError = (AlertDescription.illegal_parameter,
                                     "Suspicious A value")
            #Calculate u
            u = makeU(N, A, B)

            #Calculate premaster secret
            S = powMod((A * powMod(v,u,N)) % N, b, N)
            premasterSecret = numberToBytes(S)


        #If we've selected an RSA suite, exchange keys and calculate
        #premaster secret:
        elif cipherSuite in CipherSuite.rsaSuites:

            #Send ServerHello, Certificate[, CertificateRequest],
            #ServerHelloDone
            msgs = []
            msgs.append(ServerHello().create(self.version, serverRandom,
                        sessionID, cipherSuite, certificateType))
            msgs.append(Certificate(certificateType).create(serverCertChain))
            if reqCert:
                msgs.append(CertificateRequest())
            msgs.append(ServerHelloDone())
            for result in self._sendMsgs(msgs):
                yield result

            #From here on, the client's messages must have the right version
            self._versionCheck = True

            #Get [Certificate,] (if was requested)
            if reqCert:
                if self.version == (3,0):
                    for result in self._getMsg((ContentType.handshake,
                                               ContentType.alert),
                                               HandshakeType.certificate,
                                               certificateType):
                        if result in (0,1):
                            yield result
                        else:
                            break
                    msg = result

                    if isinstance(msg, Alert):
                        #If it's not a no_certificate alert, re-raise
                        alert = msg
                        if alert.description != \
                                AlertDescription.no_certificate:
                            self._shutdown(False)
                            raise TLSRemoteAlert(alert)
                    elif isinstance(msg, Certificate):
                        clientCertificate = msg
                        if clientCertificate.certChain and \
                                clientCertificate.certChain.getNumCerts()!=0:
                            clientCertChain = clientCertificate.certChain
                    else:
                        raise AssertionError()
                elif self.version in ((3,1), (3,2)):
                    for result in self._getMsg(ContentType.handshake,
                                              HandshakeType.certificate,
                                              certificateType):
                        if result in (0,1):
                            yield result
                        else:
                            break
                    clientCertificate = result
                    if clientCertificate.certChain and \
                            clientCertificate.certChain.getNumCerts()!=0:
                        clientCertChain = clientCertificate.certChain
                else:
                    raise AssertionError()

            #Get ClientKeyExchange
            for result in self._getMsg(ContentType.handshake,
                                      HandshakeType.client_key_exchange,
                                      cipherSuite):
                if result in (0,1):
                    yield result
                else:
                    break
            clientKeyExchange = result

            #Decrypt ClientKeyExchange
            premasterSecret = privateKey.decrypt(\
                clientKeyExchange.encryptedPreMasterSecret)

            randomPreMasterSecret = getRandomBytes(48)
            versionCheck = (premasterSecret[0], premasterSecret[1])
            if not premasterSecret:
                premasterSecret = randomPreMasterSecret
            elif len(premasterSecret)!=48:
                premasterSecret = randomPreMasterSecret
            elif versionCheck != clientHello.client_version:
                if versionCheck != self.version: #Tolerate buggy IE clients
                    premasterSecret = randomPreMasterSecret

            #Get and check CertificateVerify, if relevant
            if clientCertChain:
                if self.version == (3,0):
                    #Create a temporary session object, just for the purpose
                    #of checking the CertificateVerify
                    session = Session()
                    session._calcMasterSecret(self.version, premasterSecret,
                                             clientRandom, serverRandom)
                    verifyBytes = self._calcSSLHandshakeHash(\
                                    session.masterSecret, "")
                elif self.version in ((3,1), (3,2)):
                    verifyBytes = stringToBytes(self._handshake_md5.digest() +\
                                                self._handshake_sha.digest())
                for result in self._getMsg(ContentType.handshake,
                                          HandshakeType.certificate_verify):
                    if result in (0,1):
                        yield result
                    else:
                        break
                certificateVerify = result
                publicKey = clientCertChain.getEndEntityPublicKey()
                if len(publicKey) < settings.minKeySize:
                    postFinishedError = (AlertDescription.handshake_failure,
                        "Client's public key too small: %d" % len(publicKey))
                if len(publicKey) > settings.maxKeySize:
                    postFinishedError = (AlertDescription.handshake_failure,
                        "Client's public key too large: %d" % len(publicKey))

                if not publicKey.verify(certificateVerify.signature,
                                        verifyBytes):
                    postFinishedError = (AlertDescription.decrypt_error,
                                         "Signature failed to verify")


        #Create the session object
        self.session = Session()
        self.session._calcMasterSecret(self.version, premasterSecret,
                                      clientRandom, serverRandom)
        self.session.sessionID = sessionID
        self.session.cipherSuite = cipherSuite
        self.session.srpUsername = self.allegedSrpUsername
        self.session.clientCertChain = clientCertChain
        self.session.serverCertChain = serverCertChain

        #Calculate pending connection states
        self._calcPendingStates(clientRandom, serverRandom,
                               settings.cipherImplementations)

        #Exchange ChangeCipherSpec and Finished messages
        for result in self._getFinished():
            yield result

        #If we were holding a post-finished error until receiving the client
        #finished message, send it now.  We delay the call until this point
        #because calling sendError() throws an exception, and our caller might
        #shut down the socket upon receiving the exception.  If he did, and the
        #client was still sending its ChangeCipherSpec or Finished messages, it
        #would cause a socket error on the client side.  This is a lot of
        #consideration to show to misbehaving clients, but this would also
        #cause problems with fault-testing.
        if postFinishedError:
            for result in self._sendError(*postFinishedError):
                yield result

        for result in self._sendFinished():
            yield result

        #Add the session object to the session cache
        if sessionCache and sessionID:
            sessionCache[bytesToString(sessionID)] = self.session

        #Mark the connection as open
        self.session._setResumable(True)
        self._handshakeDone(resumed=False)


    def _handshakeWrapperAsync(self, handshaker, checker):
        if not self.fault:
            try:
                for result in handshaker:
                    yield result
                if checker:
                    try:
                        checker(self)
                    except TLSAuthenticationError:
                        alert = Alert().create(AlertDescription.close_notify,
                                               AlertLevel.fatal)
                        for result in self._sendMsg(alert):
                            yield result
                        raise
            except:
                    self._shutdown(False)
                    raise
        else:
            try:
                for result in handshaker:
                    yield result
                if checker:
                    try:
                        checker(self)
                    except TLSAuthenticationError:
                        alert = Alert().create(AlertDescription.close_notify,
                                               AlertLevel.fatal)
                        for result in self._sendMsg(alert):
                            yield result
                        raise
            except socket.error, e:
                raise TLSFaultError("socket error!")
            except TLSAbruptCloseError, e:
                raise TLSFaultError("abrupt close error!")
            except TLSAlert, alert:
                if alert.description not in Fault.faultAlerts[self.fault]:
                    raise TLSFaultError(str(alert))
                else:
                    pass
            except:
                self._shutdown(False)
                raise
            else:
                raise TLSFaultError("No error!")


    def _getKeyFromChain(self, certificate, settings):
        #Get and check cert chain from the Certificate message
        certChain = certificate.certChain
        if not certChain or certChain.getNumCerts() == 0:
            for result in self._sendError(AlertDescription.illegal_parameter,
                    "Other party sent a Certificate message without "\
                    "certificates"):
                yield result

        #Get and check public key from the cert chain
        publicKey = certChain.getEndEntityPublicKey()
        if len(publicKey) < settings.minKeySize:
            for result in self._sendError(AlertDescription.handshake_failure,
                    "Other party's public key too small: %d" % len(publicKey)):
                yield result
        if len(publicKey) > settings.maxKeySize:
            for result in self._sendError(AlertDescription.handshake_failure,
                    "Other party's public key too large: %d" % len(publicKey)):
                yield result

        yield publicKey, certChain



#!/usr/bin/python
#
# Copyright (C) 2008 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Provides HTTP functions for gdata.service to use on Google App Engine

AppEngineHttpClient: Provides an HTTP request method which uses App Engine's
   urlfetch API. Set the http_client member of a GDataService object to an
   instance of an AppEngineHttpClient to allow the gdata library to run on
   Google App Engine.

run_on_appengine: Function which will modify an existing GDataService object
   to allow it to run on App Engine. It works by creating a new instance of
   the AppEngineHttpClient and replacing the GDataService object's 
   http_client.

HttpRequest: Function that wraps google.appengine.api.urlfetch.Fetch in a 
    common interface which is used by gdata.service.GDataService. In other 
    words, this module can be used as the gdata service request handler so 
    that all HTTP requests will be performed by the hosting Google App Engine
    server. 
"""


__author__ = 'api.jscudder (Jeff Scudder)'


import StringIO
import atom.service
import atom.http_interface
from google.appengine.api import urlfetch


def run_on_appengine(gdata_service):
  """Modifies a GDataService object to allow it to run on App Engine.

  Args:
    gdata_service: An instance of AtomService, GDataService, or any
        of their subclasses which has an http_client member.
  """
  gdata_service.http_client = AppEngineHttpClient()


class AppEngineHttpClient(atom.http_interface.GenericHttpClient):
  def __init__(self, headers=None):
    self.debug = False
    self.headers = headers or {}

  def request(self, operation, url, data=None, headers=None):
    """Performs an HTTP call to the server, supports GET, POST, PUT, and
    DELETE.

    Usage example, perform and HTTP GET on http://www.google.com/:
      import atom.http
      client = atom.http.HttpClient()
      http_response = client.request('GET', 'http://www.google.com/')

    Args:
      operation: str The HTTP operation to be performed. This is usually one
          of 'GET', 'POST', 'PUT', or 'DELETE'
      data: filestream, list of parts, or other object which can be converted
          to a string. Should be set to None when performing a GET or DELETE.
          If data is a file-like object which can be read, this method will
          read a chunk of 100K bytes at a time and send them.
          If the data is a list of parts to be sent, each part will be
          evaluated and sent.
      url: The full URL to which the request should be sent. Can be a string
          or atom.url.Url.
      headers: dict of strings. HTTP headers which should be sent
          in the request.
    """
    all_headers = self.headers.copy()
    if headers:
      all_headers.update(headers)

    # Construct the full payload.
    # Assume that data is None or a string.
    data_str = data
    if data:
      if isinstance(data, list):
        # If data is a list of different objects, convert them all to strings
        # and join them together.
        converted_parts = [__ConvertDataPart(x) for x in data]
        data_str = ''.join(converted_parts)
      else:
        data_str = __ConvertDataPart(data)

    # If the list of headers does not include a Content-Length, attempt to
    # calculate it based on the data object.
    if data and 'Content-Length' not in all_headers:
      all_headers['Content-Length'] = len(data_str)

    # Set the content type to the default value if none was set.
    if 'Content-Type' not in all_headers:
      all_headers['Content-Type'] = 'application/atom+xml'

    # Lookup the urlfetch operation which corresponds to the desired HTTP verb.
    if operation == 'GET':
      method = urlfetch.GET
    elif operation == 'POST':
      method = urlfetch.POST
    elif operation == 'PUT':
      method = urlfetch.PUT
    elif operation == 'DELETE':
      method = urlfetch.DELETE
    else:
      method = None
    return HttpResponse(urlfetch.Fetch(url=str(url), payload=data_str,
        method=method, headers=all_headers))
 

def HttpRequest(service, operation, data, uri, extra_headers=None,
    url_params=None, escape_params=True, content_type='application/atom+xml'):
  """Performs an HTTP call to the server, supports GET, POST, PUT, and DELETE.

  This function is deprecated, use AppEngineHttpClient.request instead.

  To use this module with gdata.service, you can set this module to be the
  http_request_handler so that HTTP requests use Google App Engine's urlfetch.
  import gdata.service
  import gdata.urlfetch
  gdata.service.http_request_handler = gdata.urlfetch

  Args:
    service: atom.AtomService object which contains some of the parameters
        needed to make the request. The following members are used to
        construct the HTTP call: server (str), additional_headers (dict),
        port (int), and ssl (bool).
    operation: str The HTTP operation to be performed. This is usually one of
        'GET', 'POST', 'PUT', or 'DELETE'
    data: filestream, list of parts, or other object which can be
        converted to a string.
        Should be set to None when performing a GET or PUT.
        If data is a file-like object which can be read, this method will read
        a chunk of 100K bytes at a time and send them.
        If the data is a list of parts to be sent, each part will be evaluated
        and sent.
    uri: The beginning of the URL to which the request should be sent.
        Examples: '/', '/base/feeds/snippets',
        '/m8/feeds/contacts/default/base'
    extra_headers: dict of strings. HTTP headers which should be sent
        in the request. These headers are in addition to those stored in
        service.additional_headers.
    url_params: dict of strings. Key value pairs to be added to the URL as
        URL parameters. For example {'foo':'bar', 'test':'param'} will
        become ?foo=bar&test=param.
    escape_params: bool default True. If true, the keys and values in
        url_params will be URL escaped when the form is constructed
        (Special characters converted to %XX form.)
    content_type: str The MIME type for the data being sent. Defaults to
        'application/atom+xml', this is only used if data is set.
  """
  full_uri = atom.service.BuildUri(uri, url_params, escape_params)
  (server, port, ssl, partial_uri) = atom.service.ProcessUrl(service, full_uri)
  # Construct the full URL for the request.
  if ssl:
    full_url = 'https://%s%s' % (server, partial_uri)
  else:
    full_url = 'http://%s%s' % (server, partial_uri)

  # Construct the full payload. 
  # Assume that data is None or a string.
  data_str = data
  if data:
    if isinstance(data, list):
      # If data is a list of different objects, convert them all to strings
      # and join them together.
      converted_parts = [__ConvertDataPart(x) for x in data]
      data_str = ''.join(converted_parts)
    else:
      data_str = __ConvertDataPart(data)

  # Construct the dictionary of HTTP headers.
  headers = {}
  if isinstance(service.additional_headers, dict):
    headers = service.additional_headers.copy()
  if isinstance(extra_headers, dict):
    for header, value in extra_headers.iteritems():
      headers[header] = value
  # Add the content type header (we don't need to calculate content length,
  # since urlfetch.Fetch will calculate for us).
  if content_type:
    headers['Content-Type'] = content_type

  # Lookup the urlfetch operation which corresponds to the desired HTTP verb.
  if operation == 'GET':
    method = urlfetch.GET
  elif operation == 'POST':
    method = urlfetch.POST
  elif operation == 'PUT':
    method = urlfetch.PUT
  elif operation == 'DELETE':
    method = urlfetch.DELETE
  else:
    method = None
  return HttpResponse(urlfetch.Fetch(url=full_url, payload=data_str, 
      method=method, headers=headers))


def __ConvertDataPart(data):
  if not data or isinstance(data, str):
    return data
  elif hasattr(data, 'read'):
    # data is a file like object, so read it completely.
    return data.read()
  # The data object was not a file.
  # Try to convert to a string and send the data.
  return str(data)


class HttpResponse(object):
  """Translates a urlfetch resoinse to look like an hhtplib resoinse.
  
  Used to allow the resoinse from HttpRequest to be usable by gdata.service
  methods.
  """

  def __init__(self, urlfetch_response):
    self.body = StringIO.StringIO(urlfetch_response.content)
    self.headers = urlfetch_response.headers
    self.status = urlfetch_response.status_code
    self.reason = ''

  def read(self, length=None):
    if not length:
      return self.body.read()
    else:
      return self.body.read(length)

  def getheader(self, name):
    if not self.headers.has_key(name):
      return self.headers[name.lower()]
    return self.headers[name]
    



#!/usr/bin/python
#
# Copyright (C) 2006 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Contains classes representing Google Data elements.

  Extends Atom classes to add Google Data specific elements.
"""


__author__ = 'j.s@google.com (Jeffrey Scudder)'

import os
import atom
try:
  from xml.etree import cElementTree as ElementTree
except ImportError:
  try:
    import cElementTree as ElementTree
  except ImportError:
    try:
      from xml.etree import ElementTree
    except ImportError:
      from elementtree import ElementTree


# XML namespaces which are often used in GData entities.
GDATA_NAMESPACE = 'http://schemas.google.com/g/2005'
GDATA_TEMPLATE = '{http://schemas.google.com/g/2005}%s'
OPENSEARCH_NAMESPACE = 'http://a9.com/-/spec/opensearchrss/1.0/'
OPENSEARCH_TEMPLATE = '{http://a9.com/-/spec/opensearchrss/1.0/}%s'
BATCH_NAMESPACE = 'http://schemas.google.com/gdata/batch'
GACL_NAMESPACE = 'http://schemas.google.com/acl/2007'
GACL_TEMPLATE = '{http://schemas.google.com/acl/2007}%s'


# Labels used in batch request entries to specify the desired CRUD operation.
BATCH_INSERT = 'insert'
BATCH_UPDATE = 'update'
BATCH_DELETE = 'delete'
BATCH_QUERY = 'query'

class Error(Exception):
  pass


class MissingRequiredParameters(Error):
  pass


class MediaSource(object):
  """GData Entries can refer to media sources, so this class provides a
  place to store references to these objects along with some metadata.
  """

  def __init__(self, file_handle=None, content_type=None, content_length=None,
      file_path=None, file_name=None):
    """Creates an object of type MediaSource.

    Args:
      file_handle: A file handle pointing to the file to be encapsulated in the
                   MediaSource
      content_type: string The MIME type of the file. Required if a file_handle
                    is given.
      content_length: int The size of the file. Required if a file_handle is
                      given.
      file_path: string (optional) A full path name to the file. Used in
                    place of a file_handle.
      file_name: string The name of the file without any path information.
                 Required if a file_handle is given.
    """
    self.file_handle = file_handle
    self.content_type = content_type
    self.content_length = content_length
    self.file_name = file_name

    if (file_handle is None and content_type is not None and
        file_path is not None):
      self.setFile(file_path, content_type)

  def setFile(self, file_name, content_type):
    """A helper function which can create a file handle from a given filename
    and set the content type and length all at once.

    Args:
      file_name: string The path and file name to the file containing the media
      content_type: string A MIME type representing the type of the media
    """

    self.file_handle = open(file_name, 'rb')
    self.content_type = content_type
    self.content_length = os.path.getsize(file_name)
    self.file_name = os.path.basename(file_name)


class LinkFinder(atom.LinkFinder):
  """An "interface" providing methods to find link elements

  GData Entry elements often contain multiple links which differ in the rel
  attribute or content type. Often, developers are interested in a specific
  type of link so this class provides methods to find specific classes of
  links.

  This class is used as a mixin in GData entries.
  """

  def GetSelfLink(self):
    """Find the first link with rel set to 'self'

    Returns:
      An atom.Link or none if none of the links had rel equal to 'self'
    """

    for a_link in self.link:
      if a_link.rel == 'self':
        return a_link
    return None

  def GetEditLink(self):
    for a_link in self.link:
      if a_link.rel == 'edit':
        return a_link
    return None

  def GetEditMediaLink(self):
    """The Picasa API mistakenly returns media-edit rather than edit-media, but
    this may change soon.
    """
    for a_link in self.link:
      if a_link.rel == 'edit-media':
        return a_link
      if a_link.rel == 'media-edit':
        return a_link
    return None

  def GetHtmlLink(self):
    """Find the first link with rel of alternate and type of text/html

    Returns:
      An atom.Link or None if no links matched
    """
    for a_link in self.link:
      if a_link.rel == 'alternate' and a_link.type == 'text/html':
        return a_link
    return None

  def GetPostLink(self):
    """Get a link containing the POST target URL.

    The POST target URL is used to insert new entries.

    Returns:
      A link object with a rel matching the POST type.
    """
    for a_link in self.link:
      if a_link.rel == 'http://schemas.google.com/g/2005#post':
        return a_link
    return None

  def GetAclLink(self):
    for a_link in self.link:
      if a_link.rel == 'http://schemas.google.com/acl/2007#accessControlList':
        return a_link
    return None

  def GetFeedLink(self):
    for a_link in self.link:
      if a_link.rel == 'http://schemas.google.com/g/2005#feed':
        return a_link
    return None

  def GetNextLink(self):
    for a_link in self.link:
      if a_link.rel == 'next':
        return a_link
    return None

  def GetPrevLink(self):
    for a_link in self.link:
      if a_link.rel == 'previous':
        return a_link
    return None


class TotalResults(atom.AtomBase):
  """opensearch:TotalResults for a GData feed"""

  _tag = 'totalResults'
  _namespace = OPENSEARCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()

  def __init__(self, extension_elements=None,
     extension_attributes=None, text=None):
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def TotalResultsFromString(xml_string):
  return atom.CreateClassFromXMLString(TotalResults, xml_string)


class StartIndex(atom.AtomBase):
  """The opensearch:startIndex element in GData feed"""

  _tag = 'startIndex'
  _namespace = OPENSEARCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()

  def __init__(self, extension_elements=None,
      extension_attributes=None, text=None):
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def StartIndexFromString(xml_string):
  return atom.CreateClassFromXMLString(StartIndex, xml_string)


class ItemsPerPage(atom.AtomBase):
  """The opensearch:itemsPerPage element in GData feed"""

  _tag = 'itemsPerPage'
  _namespace = OPENSEARCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()

  def __init__(self, extension_elements=None,
      extension_attributes=None, text=None):
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def ItemsPerPageFromString(xml_string):
  return atom.CreateClassFromXMLString(ItemsPerPage, xml_string)


class ExtendedProperty(atom.AtomBase):
  """The Google Data extendedProperty element.

  Used to store arbitrary key-value information specific to your
  application. The value can either be a text string stored as an XML
  attribute (.value), or an XML node (XmlBlob) as a child element.

  This element is used in the Google Calendar data API and the Google
  Contacts data API.
  """

  _tag = 'extendedProperty'
  _namespace = GDATA_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['name'] = 'name'
  _attributes['value'] = 'value'

  def __init__(self, name=None, value=None, extension_elements=None,
      extension_attributes=None, text=None):
    self.name = name
    self.value = value
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}

  def GetXmlBlobExtensionElement(self):
    """Returns the XML blob as an atom.ExtensionElement.

    Returns:
      An atom.ExtensionElement representing the blob's XML, or None if no
      blob was set.
    """
    if len(self.extension_elements) < 1:
      return None
    else:
      return self.extension_elements[0]

  def GetXmlBlobString(self):
    """Returns the XML blob as a string.

    Returns:
      A string containing the blob's XML, or None if no blob was set.
    """
    blob = self.GetXmlBlobExtensionElement()
    if blob:
      return blob.ToString()
    return None

  def SetXmlBlob(self, blob):
    """Sets the contents of the extendedProperty to XML as a child node.

    Since the extendedProperty is only allowed one child element as an XML
    blob, setting the XML blob will erase any preexisting extension elements
    in this object.

    Args:
      blob: str, ElementTree Element or atom.ExtensionElement representing
            the XML blob stored in the extendedProperty.
    """
    # Erase any existing extension_elements, clears the child nodes from the
    # extendedProperty.
    self.extension_elements = []
    if isinstance(blob, atom.ExtensionElement):
      self.extension_elements.append(blob)
    elif ElementTree.iselement(blob):
      self.extension_elements.append(atom._ExtensionElementFromElementTree(
          blob))
    else:
      self.extension_elements.append(atom.ExtensionElementFromString(blob))


def ExtendedPropertyFromString(xml_string):
  return atom.CreateClassFromXMLString(ExtendedProperty, xml_string)


class GDataEntry(atom.Entry, LinkFinder):
  """Extends Atom Entry to provide data processing"""

  _tag = atom.Entry._tag
  _namespace = atom.Entry._namespace
  _children = atom.Entry._children.copy()
  _attributes = atom.Entry._attributes.copy()

  def __GetId(self):
    return self.__id

  # This method was created to strip the unwanted whitespace from the id's
  # text node.
  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def IsMedia(self):
    """Determines whether or not an entry is a GData Media entry.
    """
    if (self.GetEditMediaLink()):
      return True
    else:
      return False

  def GetMediaURL(self):
    """Returns the URL to the media content, if the entry is a media entry.
    Otherwise returns None.
    """
    if not self.IsMedia():
      return None
    else:
      return self.content.src


def GDataEntryFromString(xml_string):
  """Creates a new GDataEntry instance given a string of XML."""
  return atom.CreateClassFromXMLString(GDataEntry, xml_string)


class GDataFeed(atom.Feed, LinkFinder):
  """A Feed from a GData service"""

  _tag = 'feed'
  _namespace = atom.ATOM_NAMESPACE
  _children = atom.Feed._children.copy()
  _attributes = atom.Feed._attributes.copy()
  _children['{%s}totalResults' % OPENSEARCH_NAMESPACE] = ('total_results',
                                                          TotalResults)
  _children['{%s}startIndex' % OPENSEARCH_NAMESPACE] = ('start_index',
                                                        StartIndex)
  _children['{%s}itemsPerPage' % OPENSEARCH_NAMESPACE] = ('items_per_page',
                                                          ItemsPerPage)
               # Add a conversion rule for atom:entry to make it into a GData
               # Entry.
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [GDataEntry])

  def __GetId(self):
    return self.__id

  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def __GetGenerator(self):
    return self.__generator

  def __SetGenerator(self, generator):
    self.__generator = generator
    if generator is not None:
      self.__generator.text = generator.text.strip()

  generator = property(__GetGenerator, __SetGenerator)

  def __init__(self, author=None, category=None, contributor=None,
      generator=None, icon=None, atom_id=None, link=None, logo=None,
      rights=None, subtitle=None, title=None, updated=None, entry=None,
      total_results=None, start_index=None, items_per_page=None,
      extension_elements=None, extension_attributes=None, text=None):
    """Constructor for Source

    Args:
      author: list (optional) A list of Author instances which belong to this
          class.
      category: list (optional) A list of Category instances
      contributor: list (optional) A list on Contributor instances
      generator: Generator (optional)
      icon: Icon (optional)
      id: Id (optional) The entry's Id element
      link: list (optional) A list of Link instances
      logo: Logo (optional)
      rights: Rights (optional) The entry's Rights element
      subtitle: Subtitle (optional) The entry's subtitle element
      title: Title (optional) the entry's title element
      updated: Updated (optional) the entry's updated element
      entry: list (optional) A list of the Entry instances contained in the
          feed.
      text: String (optional) The text contents of the element. This is the
          contents of the Entry's XML text node.
          (Example: <foo>This is the text</foo>)
      extension_elements: list (optional) A list of ExtensionElement instances
          which are children of this element.
      extension_attributes: dict (optional) A dictionary of strings which are
          the values for additional XML attributes of this element.
    """

    self.author = author or []
    self.category = category or []
    self.contributor = contributor or []
    self.generator = generator
    self.icon = icon
    self.id = atom_id
    self.link = link or []
    self.logo = logo
    self.rights = rights
    self.subtitle = subtitle
    self.title = title
    self.updated = updated
    self.entry = entry or []
    self.total_results = total_results
    self.start_index = start_index
    self.items_per_page = items_per_page
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def GDataFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(GDataFeed, xml_string)


class BatchId(atom.AtomBase):
  _tag = 'id'
  _namespace = BATCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()


def BatchIdFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchId, xml_string)


class BatchOperation(atom.AtomBase):
  _tag = 'operation'
  _namespace = BATCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['type'] = 'type'

  def __init__(self, op_type=None, extension_elements=None,
               extension_attributes=None,
               text=None):
    self.type = op_type
    atom.AtomBase.__init__(self,
                           extension_elements=extension_elements,
                           extension_attributes=extension_attributes,
                           text=text)


def BatchOperationFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchOperation, xml_string)


class BatchStatus(atom.AtomBase):
  """The batch:status element present in a batch response entry.

  A status element contains the code (HTTP response code) and
  reason as elements. In a single request these fields would
  be part of the HTTP response, but in a batch request each
  Entry operation has a corresponding Entry in the response
  feed which includes status information.

  See http://code.google.com/apis/gdata/batch.html#Handling_Errors
  """

  _tag = 'status'
  _namespace = BATCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['code'] = 'code'
  _attributes['reason'] = 'reason'
  _attributes['content-type'] = 'content_type'

  def __init__(self, code=None, reason=None, content_type=None,
               extension_elements=None, extension_attributes=None, text=None):
    self.code = code
    self.reason = reason
    self.content_type = content_type
    atom.AtomBase.__init__(self, extension_elements=extension_elements,
                           extension_attributes=extension_attributes,
                           text=text)


def BatchStatusFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchStatus, xml_string)


class BatchEntry(GDataEntry):
  """An atom:entry for use in batch requests.

  The BatchEntry contains additional members to specify the operation to be
  performed on this entry and a batch ID so that the server can reference
  individual operations in the response feed. For more information, see:
  http://code.google.com/apis/gdata/batch.html
  """

  _tag = GDataEntry._tag
  _namespace = GDataEntry._namespace
  _children = GDataEntry._children.copy()
  _children['{%s}operation' % BATCH_NAMESPACE] = ('batch_operation', BatchOperation)
  _children['{%s}id' % BATCH_NAMESPACE] = ('batch_id', BatchId)
  _children['{%s}status' % BATCH_NAMESPACE] = ('batch_status', BatchStatus)
  _attributes = GDataEntry._attributes.copy()

  def __init__(self, author=None, category=None, content=None,
      contributor=None, atom_id=None, link=None, published=None, rights=None,
      source=None, summary=None, control=None, title=None, updated=None,
      batch_operation=None, batch_id=None, batch_status=None,
      extension_elements=None, extension_attributes=None, text=None):
    self.batch_operation = batch_operation
    self.batch_id = batch_id
    self.batch_status = batch_status
    GDataEntry.__init__(self, author=author, category=category,
        content=content, contributor=contributor, atom_id=atom_id, link=link,
        published=published, rights=rights, source=source, summary=summary,
        control=control, title=title, updated=updated,
        extension_elements=extension_elements,
        extension_attributes=extension_attributes, text=text)


def BatchEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchEntry, xml_string)


class BatchInterrupted(atom.AtomBase):
  """The batch:interrupted element sent if batch request was interrupted.

  Only appears in a feed if some of the batch entries could not be processed.
  See: http://code.google.com/apis/gdata/batch.html#Handling_Errors
  """

  _tag = 'interrupted'
  _namespace = BATCH_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['reason'] = 'reason'
  _attributes['success'] = 'success'
  _attributes['failures'] = 'failures'
  _attributes['parsed'] = 'parsed'

  def __init__(self, reason=None, success=None, failures=None, parsed=None,
               extension_elements=None, extension_attributes=None, text=None):
    self.reason = reason
    self.success = success
    self.failures = failures
    self.parsed = parsed
    atom.AtomBase.__init__(self, extension_elements=extension_elements,
                           extension_attributes=extension_attributes,
                           text=text)


def BatchInterruptedFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchInterrupted, xml_string)


class BatchFeed(GDataFeed):
  """A feed containing a list of batch request entries."""

  _tag = GDataFeed._tag
  _namespace = GDataFeed._namespace
  _children = GDataFeed._children.copy()
  _attributes = GDataFeed._attributes.copy()
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [BatchEntry])
  _children['{%s}interrupted' % BATCH_NAMESPACE] = ('interrupted', BatchInterrupted)

  def __init__(self, author=None, category=None, contributor=None,
      generator=None, icon=None, atom_id=None, link=None, logo=None,
      rights=None, subtitle=None, title=None, updated=None, entry=None,
      total_results=None, start_index=None, items_per_page=None,
      interrupted=None,
      extension_elements=None, extension_attributes=None, text=None):
    self.interrupted = interrupted
    GDataFeed.__init__(self, author=author, category=category,
                       contributor=contributor, generator=generator,
                       icon=icon, atom_id=atom_id, link=link,
                       logo=logo, rights=rights, subtitle=subtitle,
                       title=title, updated=updated, entry=entry,
                       total_results=total_results, start_index=start_index,
                       items_per_page=items_per_page,
                       extension_elements=extension_elements,
                       extension_attributes=extension_attributes,
                       text=text)

  def AddBatchEntry(self, entry=None, id_url_string=None,
                     batch_id_string=None, operation_string=None):
    """Logic for populating members of a BatchEntry and adding to the feed.


    If the entry is not a BatchEntry, it is converted to a BatchEntry so
    that the batch specific members will be present.

    The id_url_string can be used in place of an entry if the batch operation
    applies to a URL. For example query and delete operations require just
    the URL of an entry, no body is sent in the HTTP request. If an
    id_url_string is sent instead of an entry, a BatchEntry is created and
    added to the feed.

    This method also assigns the desired batch id to the entry so that it
    can be referenced in the server's response. If the batch_id_string is
    None, this method will assign a batch_id to be the index at which this
    entry will be in the feed's entry list.

    Args:
      entry: BatchEntry, atom.Entry, or another Entry flavor (optional) The
          entry which will be sent to the server as part of the batch request.
          The item must have a valid atom id so that the server knows which
          entry this request references.
      id_url_string: str (optional) The URL of the entry to be acted on. You
          can find this URL in the text member of the atom id for an entry.
          If an entry is not sent, this id will be used to construct a new
          BatchEntry which will be added to the request feed.
      batch_id_string: str (optional) The batch ID to be used to reference
          this batch operation in the results feed. If this parameter is None,
          the current length of the feed's entry array will be used as a
          count. Note that batch_ids should either always be specified or
          never, mixing could potentially result in duplicate batch ids.
      operation_string: str (optional) The desired batch operation which will
          set the batch_operation.type member of the entry. Options are
          'insert', 'update', 'delete', and 'query'

    Raises:
      MissingRequiredParameters: Raised if neither an id_ url_string nor an
          entry are provided in the request.

    Returns:
      The added entry.
    """
    if entry is None and id_url_string is None:
      raise MissingRequiredParameters('supply either an entry or URL string')
    if entry is None and id_url_string is not None:
      entry = BatchEntry(atom_id=atom.Id(text=id_url_string))
    # TODO: handle cases in which the entry lacks batch_... members.
    #if not isinstance(entry, BatchEntry):
      # Convert the entry to a batch entry.
    if batch_id_string is not None:
      entry.batch_id = BatchId(text=batch_id_string)
    elif entry.batch_id is None or entry.batch_id.text is None:
      entry.batch_id = BatchId(text=str(len(self.entry)))
    if operation_string is not None:
      entry.batch_operation = BatchOperation(op_type=operation_string)
    self.entry.append(entry)
    return entry

  def AddInsert(self, entry, batch_id_string=None):
    """Add an insert request to the operations in this batch request feed.

    If the entry doesn't yet have an operation or a batch id, these will
    be set to the insert operation and a batch_id specified as a parameter.

    Args:
      entry: BatchEntry The entry which will be sent in the batch feed as an
          insert request.
      batch_id_string: str (optional) The batch ID to be used to reference
          this batch operation in the results feed. If this parameter is None,
          the current length of the feed's entry array will be used as a
          count. Note that batch_ids should either always be specified or
          never, mixing could potentially result in duplicate batch ids.
    """
    entry = self.AddBatchEntry(entry=entry, batch_id_string=batch_id_string,
                               operation_string=BATCH_INSERT)

  def AddUpdate(self, entry, batch_id_string=None):
    """Add an update request to the list of batch operations in this feed.

    Sets the operation type of the entry to insert if it is not already set
    and assigns the desired batch id to the entry so that it can be
    referenced in the server's response.

    Args:
      entry: BatchEntry The entry which will be sent to the server as an
          update (HTTP PUT) request. The item must have a valid atom id
          so that the server knows which entry to replace.
      batch_id_string: str (optional) The batch ID to be used to reference
          this batch operation in the results feed. If this parameter is None,
          the current length of the feed's entry array will be used as a
          count. See also comments for AddInsert.
    """
    entry = self.AddBatchEntry(entry=entry, batch_id_string=batch_id_string,
                               operation_string=BATCH_UPDATE)

  def AddDelete(self, url_string=None, entry=None, batch_id_string=None):
    """Adds a delete request to the batch request feed.

    This method takes either the url_string which is the atom id of the item
    to be deleted, or the entry itself. The atom id of the entry must be
    present so that the server knows which entry should be deleted.

    Args:
      url_string: str (optional) The URL of the entry to be deleted. You can
         find this URL in the text member of the atom id for an entry.
      entry: BatchEntry (optional) The entry to be deleted.
      batch_id_string: str (optional)

    Raises:
      MissingRequiredParameters: Raised if neither a url_string nor an entry
          are provided in the request.
    """
    entry = self.AddBatchEntry(entry=entry, id_url_string=url_string,
                               batch_id_string=batch_id_string,
                               operation_string=BATCH_DELETE)

  def AddQuery(self, url_string=None, entry=None, batch_id_string=None):
    """Adds a query request to the batch request feed.

    This method takes either the url_string which is the query URL
    whose results will be added to the result feed. The query URL will
    be encapsulated in a BatchEntry, and you may pass in the BatchEntry
    with a query URL instead of sending a url_string.

    Args:
      url_string: str (optional)
      entry: BatchEntry (optional)
      batch_id_string: str (optional)

    Raises:
      MissingRequiredParameters
    """
    entry = self.AddBatchEntry(entry=entry, id_url_string=url_string,
                               batch_id_string=batch_id_string,
                               operation_string=BATCH_QUERY)

  def GetBatchLink(self):
    for link in self.link:
      if link.rel == 'http://schemas.google.com/g/2005#batch':
        return link
    return None


def BatchFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(BatchFeed, xml_string)


class EntryLink(atom.AtomBase):
  """The gd:entryLink element"""

  _tag = 'entryLink'
  _namespace = GDATA_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  # The entry used to be an atom.Entry, now it is a GDataEntry.
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', GDataEntry)
  _attributes['rel'] = 'rel'
  _attributes['readOnly'] = 'read_only'
  _attributes['href'] = 'href'

  def __init__(self, href=None, read_only=None, rel=None,
      entry=None, extension_elements=None,
      extension_attributes=None, text=None):
    self.href = href
    self.read_only = read_only
    self.rel = rel
    self.entry = entry
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def EntryLinkFromString(xml_string):
  return atom.CreateClassFromXMLString(EntryLink, xml_string)


class FeedLink(atom.AtomBase):
  """The gd:feedLink element"""

  _tag = 'feedLink'
  _namespace = GDATA_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _children['{%s}feed' % atom.ATOM_NAMESPACE] = ('feed', GDataFeed)
  _attributes['rel'] = 'rel'
  _attributes['readOnly'] = 'read_only'
  _attributes['countHint'] = 'count_hint'
  _attributes['href'] = 'href'

  def __init__(self, count_hint=None, href=None, read_only=None, rel=None,
      feed=None, extension_elements=None, extension_attributes=None,
      text=None):
    self.count_hint = count_hint
    self.href = href
    self.read_only = read_only
    self.rel = rel
    self.feed = feed
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def FeedLinkFromString(xml_string):
  return atom.CreateClassFromXMLString(FeedLink, xml_string)



"""Class for storing SRP password verifiers."""

from utils.cryptomath import *
from utils.compat import *
import mathtls
from BaseDB import BaseDB

class VerifierDB(BaseDB):
    """This class represent an in-memory or on-disk database of SRP
    password verifiers.

    A VerifierDB can be passed to a server handshake to authenticate
    a client based on one of the verifiers.

    This class is thread-safe.
    """
    def __init__(self, filename=None):
        """Create a new VerifierDB instance.

        @type filename: str
        @param filename: Filename for an on-disk database, or None for
        an in-memory database.  If the filename already exists, follow
        this with a call to open().  To create a new on-disk database,
        follow this with a call to create().
        """
        BaseDB.__init__(self, filename, "verifier")

    def _getItem(self, username, valueStr):
        (N, g, salt, verifier) = valueStr.split(" ")
        N = base64ToNumber(N)
        g = base64ToNumber(g)
        salt = base64ToString(salt)
        verifier = base64ToNumber(verifier)
        return (N, g, salt, verifier)

    def __setitem__(self, username, verifierEntry):
        """Add a verifier entry to the database.

        @type username: str
        @param username: The username to associate the verifier with.
        Must be less than 256 characters in length.  Must not already
        be in the database.

        @type verifierEntry: tuple
        @param verifierEntry: The verifier entry to add.  Use
        L{tlslite.VerifierDB.VerifierDB.makeVerifier} to create a
        verifier entry.
        """
        BaseDB.__setitem__(self, username, verifierEntry)


    def _setItem(self, username, value):
        if len(username)>=256:
            raise ValueError("username too long")
        N, g, salt, verifier = value
        N = numberToBase64(N)
        g = numberToBase64(g)
        salt = stringToBase64(salt)
        verifier = numberToBase64(verifier)
        valueStr = " ".join( (N, g, salt, verifier)  )
        return valueStr

    def _checkItem(self, value, username, param):
        (N, g, salt, verifier) = value
        x = mathtls.makeX(salt, username, param)
        v = powMod(g, x, N)
        return (verifier == v)


    def makeVerifier(username, password, bits):
        """Create a verifier entry which can be stored in a VerifierDB.

        @type username: str
        @param username: The username for this verifier.  Must be less
        than 256 characters in length.

        @type password: str
        @param password: The password for this verifier.

        @type bits: int
        @param bits: This values specifies which SRP group parameters
        to use.  It must be one of (1024, 1536, 2048, 3072, 4096, 6144,
        8192).  Larger values are more secure but slower.  2048 is a
        good compromise between safety and speed.

        @rtype: tuple
        @return: A tuple which may be stored in a VerifierDB.
        """
        return mathtls.makeVerifier(username, password, bits)
    makeVerifier = staticmethod(makeVerifier)


#!/usr/bin/python2.4
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Generic class for Set/Get properties of GData Provisioning clients."""


__author__ = 'Gunjan Sharma <gunjansharma@google.com>'


import gdata.apps
import gdata.apps_property
import gdata.data


class AppsPropertyEntry(gdata.data.GDEntry):
  """Represents a  generic entry in object form."""

  property = [gdata.apps_property.AppsProperty]

  def _GetProperty(self, name):
    """Get the apps:property value with the given name.

    Args:
      name: string Name of the apps:property value to get.

    Returns:
      The apps:property value with the given name, or None if the name was
          invalid.
    """
    value = None
    for p in self.property:
      if p.name == name:
        value = p.value
        break
    return value

  def _SetProperty(self, name, value):
    """Set the apps:property value with the given name to the given value.

    Args:
      name: string Name of the apps:property value to set.
      value: string Value to give the apps:property value with the given name.
    """
    found = False
    for i in range(len(self.property)):
      if self.property[i].name == name:
        self.property[i].value = value
        found = True
        break
    if not found:
      self.property.append(
          gdata.apps_property.AppsProperty(name=name, value=value))



#!/usr/bin/python
#
# Copyright (C) 2009 Tan Swee Heng
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Contains extensions to Atom objects used with Google Finance."""


__author__ = 'thesweeheng@gmail.com'


import atom
import gdata


GD_NAMESPACE = 'http://schemas.google.com/g/2005'
GF_NAMESPACE = 'http://schemas.google.com/finance/2007'


class Money(atom.AtomBase):
  """The <gd:money> element."""
  _tag = 'money'
  _namespace = GD_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['amount'] = 'amount'
  _attributes['currencyCode'] = 'currency_code'
  
  def __init__(self, amount=None, currency_code=None, **kwargs):
    self.amount = amount
    self.currency_code = currency_code
    atom.AtomBase.__init__(self, **kwargs)

  def __str__(self):
    return "%s %s" % (self.amount, self.currency_code)


def MoneyFromString(xml_string):
  return atom.CreateClassFromXMLString(Money, xml_string)


class _Monies(atom.AtomBase):
  """An element containing multiple <gd:money> in multiple currencies."""
  _namespace = GF_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _children['{%s}money' % GD_NAMESPACE] = ('money', [Money])
  
  def __init__(self, money=None, **kwargs):
    self.money = money or []
    atom.AtomBase.__init__(self, **kwargs)

  def __str__(self):
    return " / ".join(["%s" % i for i in self.money])


class CostBasis(_Monies):
  """The <gf:costBasis> element."""
  _tag = 'costBasis'


def CostBasisFromString(xml_string):
  return atom.CreateClassFromXMLString(CostBasis, xml_string)


class DaysGain(_Monies):
  """The <gf:daysGain> element."""
  _tag = 'daysGain'


def DaysGainFromString(xml_string):
  return atom.CreateClassFromXMLString(DaysGain, xml_string)


class Gain(_Monies):
  """The <gf:gain> element."""
  _tag = 'gain'


def GainFromString(xml_string):
  return atom.CreateClassFromXMLString(Gain, xml_string)


class MarketValue(_Monies):
  """The <gf:marketValue> element."""
  _tag = 'gain'
  _tag = 'marketValue'


def MarketValueFromString(xml_string):
  return atom.CreateClassFromXMLString(MarketValue, xml_string)


class Commission(_Monies):
  """The <gf:commission> element."""
  _tag = 'commission'


def CommissionFromString(xml_string):
  return atom.CreateClassFromXMLString(Commission, xml_string)


class Price(_Monies):
  """The <gf:price> element."""
  _tag = 'price'


def PriceFromString(xml_string):
  return atom.CreateClassFromXMLString(Price, xml_string)


class Symbol(atom.AtomBase):
  """The <gf:symbol> element."""
  _tag = 'symbol'
  _namespace = GF_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['fullName'] = 'full_name'
  _attributes['exchange'] = 'exchange'
  _attributes['symbol'] = 'symbol'
  
  def __init__(self, full_name=None, exchange=None, symbol=None, **kwargs):
    self.full_name = full_name
    self.exchange = exchange
    self.symbol = symbol
    atom.AtomBase.__init__(self, **kwargs)

  def __str__(self):
    return "%s:%s (%s)" % (self.exchange, self.symbol, self.full_name)


def SymbolFromString(xml_string):
  return atom.CreateClassFromXMLString(Symbol, xml_string)


class TransactionData(atom.AtomBase):
  """The <gf:transactionData> element."""
  _tag = 'transactionData'
  _namespace = GF_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['type'] = 'type'
  _attributes['date'] = 'date'
  _attributes['shares'] = 'shares'
  _attributes['notes'] = 'notes'
  _children = atom.AtomBase._children.copy()
  _children['{%s}commission' % GF_NAMESPACE] = ('commission', Commission)
  _children['{%s}price' % GF_NAMESPACE] = ('price', Price)

  def __init__(self, type=None, date=None, shares=None,
      notes=None, commission=None, price=None, **kwargs):
    self.type = type 
    self.date = date
    self.shares = shares
    self.notes = notes
    self.commission = commission
    self.price = price
    atom.AtomBase.__init__(self, **kwargs)


def TransactionDataFromString(xml_string):
  return atom.CreateClassFromXMLString(TransactionData, xml_string)


class TransactionEntry(gdata.GDataEntry):
  """An entry of the transaction feed.

  A TransactionEntry contains TransactionData such as the transaction
  type (Buy,  Sell,  Sell Short, or  Buy to Cover), the number of units,
  the date, the price, any commission, and any notes.
  """
  _tag = 'entry'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataEntry._children.copy() 
  _children['{%s}transactionData' % GF_NAMESPACE] = (
      'transaction_data', TransactionData)

  def __init__(self, transaction_data=None, **kwargs):
    self.transaction_data = transaction_data
    gdata.GDataEntry.__init__(self, **kwargs)

  def transaction_id(self):
    return self.id.text.split("/")[-1]

  transaction_id = property(transaction_id, doc='The transaction ID.')


def TransactionEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(TransactionEntry, xml_string)


class TransactionFeed(gdata.GDataFeed):
  """A feed that lists all of the transactions that have been recorded for
  a particular position.
  
  A transaction is a collection of information about an instance of
  buying or selling a particular security. The TransactionFeed lists all
  of the transactions that have been recorded for a particular position
  as a list of TransactionEntries.
  """
  _tag = 'feed'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataFeed._children.copy()
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [TransactionEntry])


def TransactionFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(TransactionFeed, xml_string)


class TransactionFeedLink(atom.AtomBase):
  """Link to TransactionFeed embedded in PositionEntry.

  If a PositionFeed is queried with transactions='true', TransactionFeeds
  are inlined in the returned PositionEntries. These TransactionFeeds are
  accessible via TransactionFeedLink's feed attribute.
  """
  _tag = 'feedLink'
  _namespace = GD_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['href'] = 'href'
  _children = atom.AtomBase._children.copy() 
  _children['{%s}feed' % atom.ATOM_NAMESPACE] = (
      'feed', TransactionFeed)

  def __init__(self, href=None, feed=None, **kwargs):
    self.href = href
    self.feed = feed
    atom.AtomBase.__init__(self, **kwargs)


class PositionData(atom.AtomBase):
  """The <gf:positionData> element."""
  _tag = 'positionData'
  _namespace = GF_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['gainPercentage'] = 'gain_percentage'
  _attributes['return1w'] = 'return1w'
  _attributes['return4w'] = 'return4w'
  _attributes['return3m'] = 'return3m'
  _attributes['returnYTD'] = 'returnYTD'
  _attributes['return1y'] = 'return1y'
  _attributes['return3y'] = 'return3y'
  _attributes['return5y'] = 'return5y'
  _attributes['returnOverall'] = 'return_overall'
  _attributes['shares'] = 'shares'
  _children = atom.AtomBase._children.copy()
  _children['{%s}costBasis' % GF_NAMESPACE] = ('cost_basis', CostBasis)
  _children['{%s}daysGain' % GF_NAMESPACE] = ('days_gain', DaysGain)
  _children['{%s}gain' % GF_NAMESPACE] = ('gain', Gain)
  _children['{%s}marketValue' % GF_NAMESPACE] = ('market_value', MarketValue)

  def __init__(self, gain_percentage=None,
      return1w=None, return4w=None, return3m=None, returnYTD=None,
      return1y=None, return3y=None, return5y=None, return_overall=None,
      shares=None, cost_basis=None, days_gain=None,
      gain=None, market_value=None, **kwargs):
    self.gain_percentage = gain_percentage
    self.return1w = return1w
    self.return4w = return4w
    self.return3m = return3m
    self.returnYTD = returnYTD
    self.return1y = return1y
    self.return3y = return3y
    self.return5y = return5y
    self.return_overall = return_overall
    self.shares = shares
    self.cost_basis = cost_basis
    self.days_gain = days_gain
    self.gain = gain
    self.market_value = market_value
    atom.AtomBase.__init__(self, **kwargs)


def PositionDataFromString(xml_string):
  return atom.CreateClassFromXMLString(PositionData, xml_string)


class PositionEntry(gdata.GDataEntry):
  """An entry of the position feed.

  A PositionEntry contains the ticker exchange and Symbol for a stock,
  mutual fund, or other security, along with PositionData such as the
  number of units of that security that the user holds, and performance
  statistics.
  """
  _tag = 'entry'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataEntry._children.copy() 
  _children['{%s}positionData' % GF_NAMESPACE] = (
      'position_data', PositionData)
  _children['{%s}symbol' % GF_NAMESPACE] = ('symbol', Symbol)
  _children['{%s}feedLink' % GD_NAMESPACE] = (
    'feed_link', TransactionFeedLink)

  def __init__(self, position_data=None, symbol=None, feed_link=None,
      **kwargs):
    self.position_data = position_data
    self.symbol = symbol
    self.feed_link = feed_link
    gdata.GDataEntry.__init__(self, **kwargs)

  def position_title(self):
    return self.title.text

  position_title = property(position_title,
    doc='The position title as a string (i.e. position.title.text).')

  def ticker_id(self):
    return self.id.text.split("/")[-1]

  ticker_id = property(ticker_id, doc='The position TICKER ID.')

  def transactions(self):
    if self.feed_link.feed:
      return self.feed_link.feed.entry
    else:
      return None

  transactions = property(transactions, doc="""
      Inlined TransactionEntries are returned if PositionFeed is queried
      with transactions='true'.""")


def PositionEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(PositionEntry, xml_string)


class PositionFeed(gdata.GDataFeed):
  """A feed that lists all of the positions in a particular portfolio.

  A position is a collection of information about a security that the
  user holds. The PositionFeed lists all of the positions in a particular
  portfolio as a list of PositionEntries.
  """
  _tag = 'feed'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataFeed._children.copy()
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [PositionEntry])


def PositionFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(PositionFeed, xml_string)


class PositionFeedLink(atom.AtomBase):
  """Link to PositionFeed embedded in PortfolioEntry.

  If a PortfolioFeed is queried with positions='true', the PositionFeeds
  are inlined in the returned PortfolioEntries. These PositionFeeds are
  accessible via PositionFeedLink's feed attribute.
  """
  _tag = 'feedLink'
  _namespace = GD_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['href'] = 'href'
  _children = atom.AtomBase._children.copy() 
  _children['{%s}feed' % atom.ATOM_NAMESPACE] = (
      'feed', PositionFeed)

  def __init__(self, href=None, feed=None, **kwargs):
    self.href = href
    self.feed = feed
    atom.AtomBase.__init__(self, **kwargs)


class PortfolioData(atom.AtomBase):
  """The <gf:portfolioData> element."""
  _tag = 'portfolioData'
  _namespace = GF_NAMESPACE
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['currencyCode'] = 'currency_code'
  _attributes['gainPercentage'] = 'gain_percentage'
  _attributes['return1w'] = 'return1w'
  _attributes['return4w'] = 'return4w'
  _attributes['return3m'] = 'return3m'
  _attributes['returnYTD'] = 'returnYTD'
  _attributes['return1y'] = 'return1y'
  _attributes['return3y'] = 'return3y'
  _attributes['return5y'] = 'return5y'
  _attributes['returnOverall'] = 'return_overall'
  _children = atom.AtomBase._children.copy()
  _children['{%s}costBasis' % GF_NAMESPACE] = ('cost_basis', CostBasis)
  _children['{%s}daysGain' % GF_NAMESPACE] = ('days_gain', DaysGain)
  _children['{%s}gain' % GF_NAMESPACE] = ('gain', Gain)
  _children['{%s}marketValue' % GF_NAMESPACE] = ('market_value', MarketValue)

  def __init__(self, currency_code=None, gain_percentage=None,
      return1w=None, return4w=None, return3m=None, returnYTD=None,
      return1y=None, return3y=None, return5y=None, return_overall=None,
      cost_basis=None, days_gain=None, gain=None, market_value=None, **kwargs):
    self.currency_code = currency_code
    self.gain_percentage = gain_percentage
    self.return1w = return1w
    self.return4w = return4w
    self.return3m = return3m
    self.returnYTD = returnYTD
    self.return1y = return1y
    self.return3y = return3y
    self.return5y = return5y
    self.return_overall = return_overall
    self.cost_basis = cost_basis
    self.days_gain = days_gain
    self.gain = gain
    self.market_value = market_value
    atom.AtomBase.__init__(self, **kwargs)


def PortfolioDataFromString(xml_string):
  return atom.CreateClassFromXMLString(PortfolioData, xml_string)


class PortfolioEntry(gdata.GDataEntry):
  """An entry of the PortfolioFeed.

  A PortfolioEntry contains the portfolio's title along with PortfolioData
  such as currency, total market value, and overall performance statistics.
  """
  _tag = 'entry'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataEntry._children.copy() 
  _children['{%s}portfolioData' % GF_NAMESPACE] = (
      'portfolio_data', PortfolioData)
  _children['{%s}feedLink' % GD_NAMESPACE] = (
    'feed_link', PositionFeedLink)

  def __init__(self, portfolio_data=None, feed_link=None, **kwargs):
    self.portfolio_data = portfolio_data
    self.feed_link = feed_link
    gdata.GDataEntry.__init__(self, **kwargs)

  def portfolio_title(self):
    return self.title.text

  def set_portfolio_title(self, portfolio_title):
    self.title = atom.Title(text=portfolio_title, title_type='text')

  portfolio_title = property(portfolio_title,  set_portfolio_title,
      doc='The portfolio title as a string (i.e. portfolio.title.text).')

  def portfolio_id(self):
    return self.id.text.split("/")[-1]

  portfolio_id = property(portfolio_id,
      doc='The portfolio ID. Do not confuse with portfolio.id.')

  def positions(self):
    if self.feed_link.feed:
      return self.feed_link.feed.entry
    else:
      return None

  positions = property(positions, doc="""
      Inlined PositionEntries are returned if PortfolioFeed was queried
      with positions='true'.""")


def PortfolioEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(PortfolioEntry, xml_string)

    
class PortfolioFeed(gdata.GDataFeed):
  """A feed that lists all of the user's portfolios.

  A portfolio is a collection of positions that the user holds in various
  securities, plus metadata. The PortfolioFeed lists all of the user's
  portfolios as a list of PortfolioEntries.
  """
  _tag = 'feed'
  _namespace = atom.ATOM_NAMESPACE
  _children = gdata.GDataFeed._children.copy()
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [PortfolioEntry])


def PortfolioFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(PortfolioFeed, xml_string)





#!/usr/bin/python
#
# Copyright (C) 2008 Yu-Jie Lin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Contains extensions to Atom objects used with Google Webmaster Tools."""


__author__ = 'livibetter (Yu-Jie Lin)'


try:
  from xml.etree import cElementTree as ElementTree
except ImportError:
  try:
    import cElementTree as ElementTree
  except ImportError:
    try:
      from xml.etree import ElementTree
    except ImportError:
      from elementtree import ElementTree
import atom
import gdata


# XML namespaces which are often used in Google Webmaster Tools entities.
GWEBMASTERTOOLS_NAMESPACE = 'http://schemas.google.com/webmasters/tools/2007'
GWEBMASTERTOOLS_TEMPLATE = '{http://schemas.google.com/webmasters/tools/2007}%s'


class Indexed(atom.AtomBase):
  _tag = 'indexed'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def IndexedFromString(xml_string):
  return atom.CreateClassFromXMLString(Indexed, xml_string)


class Crawled(atom.Date):
  _tag = 'crawled'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def CrawledFromString(xml_string):
  return atom.CreateClassFromXMLString(Crawled, xml_string)


class GeoLocation(atom.AtomBase):
  _tag = 'geolocation'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def GeoLocationFromString(xml_string):
  return atom.CreateClassFromXMLString(GeoLocation, xml_string)


class PreferredDomain(atom.AtomBase):
  _tag = 'preferred-domain'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def PreferredDomainFromString(xml_string):
  return atom.CreateClassFromXMLString(PreferredDomain, xml_string)


class CrawlRate(atom.AtomBase):
  _tag = 'crawl-rate'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def CrawlRateFromString(xml_string):
  return atom.CreateClassFromXMLString(CrawlRate, xml_string)


class EnhancedImageSearch(atom.AtomBase):
  _tag = 'enhanced-image-search'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def EnhancedImageSearchFromString(xml_string):
  return atom.CreateClassFromXMLString(EnhancedImageSearch, xml_string)


class Verified(atom.AtomBase):
  _tag = 'verified'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def VerifiedFromString(xml_string):
  return atom.CreateClassFromXMLString(Verified, xml_string)


class VerificationMethodMeta(atom.AtomBase):
  _tag = 'meta'
  _namespace = atom.ATOM_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _attributes['name'] = 'name'
  _attributes['content'] = 'content'

  def __init__(self, text=None, name=None, content=None,
      extension_elements=None, extension_attributes=None):
    self.text = text
    self.name = name
    self.content = content
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def VerificationMethodMetaFromString(xml_string):
  return atom.CreateClassFromXMLString(VerificationMethodMeta, xml_string)


class VerificationMethod(atom.AtomBase):
  _tag = 'verification-method'
  _namespace = GWEBMASTERTOOLS_NAMESPACE
  _children = atom.Text._children.copy()
  _attributes = atom.Text._attributes.copy()
  _children['{%s}meta' % atom.ATOM_NAMESPACE] = (
    'meta', VerificationMethodMeta)
  _attributes['in-use'] = 'in_use'
  _attributes['type'] = 'type'

  def __init__(self, text=None, in_use=None, meta=None, type=None,
      extension_elements=None, extension_attributes=None):
    self.text = text
    self.in_use = in_use
    self.meta = meta
    self.type = type
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def VerificationMethodFromString(xml_string):
  return atom.CreateClassFromXMLString(VerificationMethod, xml_string)


class MarkupLanguage(atom.AtomBase):
  _tag = 'markup-language'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def MarkupLanguageFromString(xml_string):
  return atom.CreateClassFromXMLString(MarkupLanguage, xml_string)


class SitemapMobile(atom.AtomBase):
  _tag = 'sitemap-mobile'
  _namespace = GWEBMASTERTOOLS_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _children['{%s}markup-language' % GWEBMASTERTOOLS_NAMESPACE] = (
      'markup_language', [MarkupLanguage])

  def __init__(self, markup_language=None,
      extension_elements=None, extension_attributes=None, text=None):

    self.markup_language = markup_language or []
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def SitemapMobileFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapMobile, xml_string)


class SitemapMobileMarkupLanguage(atom.AtomBase):
  _tag = 'sitemap-mobile-markup-language'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapMobileMarkupLanguageFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapMobileMarkupLanguage, xml_string)


class PublicationLabel(atom.AtomBase):
  _tag = 'publication-label'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def PublicationLabelFromString(xml_string):
  return atom.CreateClassFromXMLString(PublicationLabel, xml_string)


class SitemapNews(atom.AtomBase):
  _tag = 'sitemap-news'
  _namespace = GWEBMASTERTOOLS_NAMESPACE
  _children = atom.AtomBase._children.copy()
  _attributes = atom.AtomBase._attributes.copy()
  _children['{%s}publication-label' % GWEBMASTERTOOLS_NAMESPACE] = (
      'publication_label', [PublicationLabel])

  def __init__(self, publication_label=None,
    extension_elements=None, extension_attributes=None, text=None):

    self.publication_label = publication_label or []
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def SitemapNewsFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapNews, xml_string)


class SitemapNewsPublicationLabel(atom.AtomBase):
  _tag = 'sitemap-news-publication-label'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapNewsPublicationLabelFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapNewsPublicationLabel, xml_string)


class SitemapLastDownloaded(atom.Date):
  _tag = 'sitemap-last-downloaded'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapLastDownloadedFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapLastDownloaded, xml_string)


class SitemapType(atom.AtomBase):
  _tag = 'sitemap-type'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapTypeFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapType, xml_string)


class SitemapStatus(atom.AtomBase):
  _tag = 'sitemap-status'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapStatusFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapStatus, xml_string)


class SitemapUrlCount(atom.AtomBase):
  _tag = 'sitemap-url-count'
  _namespace = GWEBMASTERTOOLS_NAMESPACE


def SitemapUrlCountFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapUrlCount, xml_string)


class LinkFinder(atom.LinkFinder):
  """An "interface" providing methods to find link elements

  SitesEntry elements often contain multiple links which differ in the rel 
  attribute or content type. Often, developers are interested in a specific
  type of link so this class provides methods to find specific classes of links.

  This class is used as a mixin in SitesEntry.
  """

  def GetSelfLink(self):
    """Find the first link with rel set to 'self'

    Returns:
      An atom.Link or none if none of the links had rel equal to 'self'
    """

    for a_link in self.link:
      if a_link.rel == 'self':
        return a_link
    return None

  def GetEditLink(self):
    for a_link in self.link:
      if a_link.rel == 'edit':
        return a_link
    return None

  def GetPostLink(self):
    """Get a link containing the POST target URL.
    
    The POST target URL is used to insert new entries.

    Returns:
      A link object with a rel matching the POST type.
    """
    for a_link in self.link:
      if a_link.rel == 'http://schemas.google.com/g/2005#post':
        return a_link
    return None

  def GetFeedLink(self):
    for a_link in self.link:
      if a_link.rel == 'http://schemas.google.com/g/2005#feed':
        return a_link
    return None


class SitesEntry(atom.Entry, LinkFinder):
  """A Google Webmaster Tools meta Entry flavor of an Atom Entry """

  _tag = atom.Entry._tag
  _namespace = atom.Entry._namespace
  _children = atom.Entry._children.copy()
  _attributes = atom.Entry._attributes.copy()
  _children['{%s}entryLink' % gdata.GDATA_NAMESPACE] = (
      'entry_link', [gdata.EntryLink])
  _children['{%s}indexed' % GWEBMASTERTOOLS_NAMESPACE] = ('indexed', Indexed)
  _children['{%s}crawled' % GWEBMASTERTOOLS_NAMESPACE] = (
      'crawled', Crawled)
  _children['{%s}geolocation' % GWEBMASTERTOOLS_NAMESPACE] = (
      'geolocation', GeoLocation)
  _children['{%s}preferred-domain' % GWEBMASTERTOOLS_NAMESPACE] = (
      'preferred_domain', PreferredDomain)
  _children['{%s}crawl-rate' % GWEBMASTERTOOLS_NAMESPACE] = (
      'crawl_rate', CrawlRate)
  _children['{%s}enhanced-image-search' % GWEBMASTERTOOLS_NAMESPACE] = (
      'enhanced_image_search', EnhancedImageSearch)
  _children['{%s}verified' % GWEBMASTERTOOLS_NAMESPACE] = (
      'verified', Verified)
  _children['{%s}verification-method' % GWEBMASTERTOOLS_NAMESPACE] = (
      'verification_method', [VerificationMethod])
  
  def __GetId(self):
    return self.__id

  # This method was created to strip the unwanted whitespace from the id's 
  # text node.
  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def __init__(self, category=None, content=None,
      atom_id=None, link=None, title=None, updated=None,
      entry_link=None, indexed=None, crawled=None,
      geolocation=None, preferred_domain=None, crawl_rate=None,
      enhanced_image_search=None,
      verified=None, verification_method=None,
      extension_elements=None, extension_attributes=None, text=None):
    atom.Entry.__init__(self, category=category, 
                        content=content, atom_id=atom_id, link=link, 
                        title=title, updated=updated, text=text)

    self.entry_link = entry_link or []
    self.indexed = indexed
    self.crawled = crawled
    self.geolocation = geolocation
    self.preferred_domain = preferred_domain
    self.crawl_rate = crawl_rate
    self.enhanced_image_search = enhanced_image_search
    self.verified = verified
    self.verification_method = verification_method or []


def SitesEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(SitesEntry, xml_string)


class SitesFeed(atom.Feed, LinkFinder):
  """A Google Webmaster Tools meta Sites feed flavor of an Atom Feed"""

  _tag = atom.Feed._tag
  _namespace = atom.Feed._namespace
  _children = atom.Feed._children.copy()
  _attributes = atom.Feed._attributes.copy()
  _children['{%s}startIndex' % gdata.OPENSEARCH_NAMESPACE] = (
      'start_index', gdata.StartIndex)
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [SitesEntry])
  del _children['{%s}generator' % atom.ATOM_NAMESPACE]
  del _children['{%s}author' % atom.ATOM_NAMESPACE]
  del _children['{%s}contributor' % atom.ATOM_NAMESPACE]
  del _children['{%s}logo' % atom.ATOM_NAMESPACE]
  del _children['{%s}icon' % atom.ATOM_NAMESPACE]
  del _children['{%s}rights' % atom.ATOM_NAMESPACE]
  del _children['{%s}subtitle' % atom.ATOM_NAMESPACE]

  def __GetId(self):
    return self.__id

  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def __init__(self, start_index=None, atom_id=None, title=None, entry=None,
      category=None, link=None, updated=None,
      extension_elements=None, extension_attributes=None, text=None):
    """Constructor for Source
    
    Args:
      category: list (optional) A list of Category instances
      id: Id (optional) The entry's Id element
      link: list (optional) A list of Link instances
      title: Title (optional) the entry's title element
      updated: Updated (optional) the entry's updated element
      entry: list (optional) A list of the Entry instances contained in the 
          feed.
      text: String (optional) The text contents of the element. This is the 
          contents of the Entry's XML text node. 
          (Example: <foo>This is the text</foo>)
      extension_elements: list (optional) A list of ExtensionElement instances
          which are children of this element.
      extension_attributes: dict (optional) A dictionary of strings which are 
          the values for additional XML attributes of this element.
    """

    self.start_index = start_index
    self.category = category or []
    self.id = atom_id
    self.link = link or []
    self.title = title
    self.updated = updated
    self.entry = entry or []
    self.text = text
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def SitesFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(SitesFeed, xml_string)


class SitemapsEntry(atom.Entry, LinkFinder):
  """A Google Webmaster Tools meta Sitemaps Entry flavor of an Atom Entry """

  _tag = atom.Entry._tag
  _namespace = atom.Entry._namespace
  _children = atom.Entry._children.copy()
  _attributes = atom.Entry._attributes.copy()
  _children['{%s}sitemap-type' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_type', SitemapType)
  _children['{%s}sitemap-status' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_status', SitemapStatus)
  _children['{%s}sitemap-last-downloaded' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_last_downloaded', SitemapLastDownloaded)
  _children['{%s}sitemap-url-count' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_url_count', SitemapUrlCount)
  _children['{%s}sitemap-mobile-markup-language' % GWEBMASTERTOOLS_NAMESPACE] \
      = ('sitemap_mobile_markup_language', SitemapMobileMarkupLanguage)
  _children['{%s}sitemap-news-publication-label' % GWEBMASTERTOOLS_NAMESPACE] \
      = ('sitemap_news_publication_label', SitemapNewsPublicationLabel)
  
  def __GetId(self):
    return self.__id

  # This method was created to strip the unwanted whitespace from the id's 
  # text node.
  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def __init__(self, category=None, content=None,
      atom_id=None, link=None, title=None, updated=None,
      sitemap_type=None, sitemap_status=None, sitemap_last_downloaded=None,
      sitemap_url_count=None, sitemap_mobile_markup_language=None,
      sitemap_news_publication_label=None,
      extension_elements=None, extension_attributes=None, text=None):
    atom.Entry.__init__(self, category=category, 
                        content=content, atom_id=atom_id, link=link, 
                        title=title, updated=updated, text=text)

    self.sitemap_type = sitemap_type
    self.sitemap_status = sitemap_status
    self.sitemap_last_downloaded = sitemap_last_downloaded
    self.sitemap_url_count = sitemap_url_count
    self.sitemap_mobile_markup_language = sitemap_mobile_markup_language
    self.sitemap_news_publication_label = sitemap_news_publication_label


def SitemapsEntryFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapsEntry, xml_string)


class SitemapsFeed(atom.Feed, LinkFinder):
  """A Google Webmaster Tools meta Sitemaps feed flavor of an Atom Feed"""

  _tag = atom.Feed._tag
  _namespace = atom.Feed._namespace
  _children = atom.Feed._children.copy()
  _attributes = atom.Feed._attributes.copy()
  _children['{%s}entry' % atom.ATOM_NAMESPACE] = ('entry', [SitemapsEntry])
  _children['{%s}sitemap-mobile' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_mobile', SitemapMobile)
  _children['{%s}sitemap-news' % GWEBMASTERTOOLS_NAMESPACE] = (
      'sitemap_news', SitemapNews)
  del _children['{%s}generator' % atom.ATOM_NAMESPACE]
  del _children['{%s}author' % atom.ATOM_NAMESPACE]
  del _children['{%s}contributor' % atom.ATOM_NAMESPACE]
  del _children['{%s}logo' % atom.ATOM_NAMESPACE]
  del _children['{%s}icon' % atom.ATOM_NAMESPACE]
  del _children['{%s}rights' % atom.ATOM_NAMESPACE]
  del _children['{%s}subtitle' % atom.ATOM_NAMESPACE]

  def __GetId(self):
    return self.__id

  def __SetId(self, id):
    self.__id = id
    if id is not None and id.text is not None:
      self.__id.text = id.text.strip()

  id = property(__GetId, __SetId)

  def __init__(self, category=None, content=None,
      atom_id=None, link=None, title=None, updated=None,
      entry=None, sitemap_mobile=None, sitemap_news=None,
      extension_elements=None, extension_attributes=None, text=None):

    self.category = category or []
    self.id = atom_id
    self.link = link or []
    self.title = title
    self.updated = updated
    self.entry = entry or []
    self.text = text
    self.sitemap_mobile = sitemap_mobile
    self.sitemap_news = sitemap_news
    self.extension_elements = extension_elements or []
    self.extension_attributes = extension_attributes or {}


def SitemapsFeedFromString(xml_string):
  return atom.CreateClassFromXMLString(SitemapsFeed, xml_string)
